/* Tables used by TradingService Attribute Database  */

CREATE TABLE OF_Attributes
(
   Name VARCHAR2 (36) NOT NULL, /* name of the attribute */
   PID NUMBER NOT NULL, /* process id of this trader */
   Object NUMBER NOT NULL, /* value of the attribute */
   PRIMARY KEY (Name, PID)
)
;

/* Tables used by TradingService Service Database */

CREATE TABLE OF_Services
(
   Incarnation NUMBER NOT NULL, /* consecutive integer id number */
   PID NUMBER NOT NULL, /* process id of this trader */
   ServiceTypeIndex NUMBER NOT NULL, /* service type name index */
   Masked NUMBER NOT NULL, /* boolean value, whether service type is masked */
   InterfaceTypeIndex NUMBER NOT NULL, /* service interface name index */
   PRIMARY KEY (Incarnation, PID)
)
;

CREATE TABLE OF_ServiceProperties
(
   Incarnation NUMBER NOT NULL, /* consecutive integer id number */
   PID NUMBER NOT NULL, /* process id of this trader */
   NameIndex NUMBER NOT NULL, /* property name index */
   PMode NUMBER NOT NULL, /* propertymode as an integer */
   TypeCode LONG RAW NOT NULL /* serialized typecode */
)
;


CREATE TABLE OF_SuperTypes
(
   Incarnation NUMBER NOT NULL, /* consecutive integer id number */
   PID NUMBER NOT NULL, /* process id of this trader */
   ServiceType NUMBER NOT NULL /* incarnation number of a supertype */
)
;

/* Tables used by TradingService Offer Database */

CREATE TABLE OF_Offers
(
   OfferID VARCHAR2 (36) NOT NULL, /* offer id (UUID) */
   PID NUMBER NOT NULL, /* process id of this trader */
   IsProxy NUMBER NOT NULL, /* boolean, true if offer is a proxy, false otherwise */
   ServiceTypeIndex NUMBER NOT NULL, /* service type name index */
   OfferData LONG RAW NOT NULL, /* stringified object reference */
   MatchAll NUMBER NOT NULL, /*boolean, true if IF_MATCH_ALL is true */
   PRIMARY KEY (OfferID)
)
;

CREATE TABLE OF_OfferProps
(
   Name VARCHAR2 (50),  /* property name */
   Value VARCHAR2 (200),        /* property value */
   ServiceTypeIndex NUMBER NOT NULL,  /* service type name index */
   PID NUMBER NOT NULL,  /* process id of this trader */
   NumVal NUMBER, /* numeric property value */
   OfferID VARCHAR (36) NOT NULL,  /* offer id ( UUID ) */
   ComplexValue NUMBER NOT NULL /* boolean, true if property value is dynamically evaluated, or is not a primative */
)
;

CREATE PROCEDURE of_insert_attribute
(
   nm IN VARCHAR2,
   pd IN NUMBER,
   obj IN NUMBER
)
IS
BEGIN
   INSERT INTO OF_Attributes
      (Name, PID, Object)
      VALUES
      (nm, pd, obj);
END of_insert_attribute;
/

CREATE PROCEDURE of_update_attribute
(
   nm IN VARCHAR2,
   pd IN NUMBER,
   obj IN NUMBER
)
IS
BEGIN
   UPDATE OF_Attributes
      SET Object = obj
      WHERE Name = nm AND PID = pd;
END of_update_attribute;
/

CREATE PROCEDURE of_restore_attribute
(
   nm IN VARCHAR2,
   pd IN NUMBER,
   obj OUT NUMBER
)
IS
BEGIN
   SELECT Object INTO obj FROM OF_Attributes
      WHERE Name = nm AND PID = pd;
EXCEPTION
      WHEN NO_DATA_FOUND
   THEN
      obj := NULL;
END of_restore_attribute;
/

CREATE PROCEDURE of_attribute_exists
(
   nm IN VARCHAR2,
   pd IN NUMBER,
   b OUT NUMBER
)
IS
   obj NUMBER;
BEGIN
   SELECT Object INTO obj FROM OF_Attributes
      WHERE Name = nm AND PID = pd;
   b := 1;
EXCEPTION
      WHEN NO_DATA_FOUND
   THEN
      b := 0;
END of_attribute_exists;
/

CREATE PROCEDURE of_servicetype
(
   inc IN NUMBER,
   pd IN NUMBER,
   idx OUT NUMBER
)
IS
BEGIN
   SELECT ServiceTypeIndex INTO idx FROM OF_Services
      WHERE Incarnation = inc AND PID = pd;
END of_servicetype;
/

CREATE PROCEDURE of_incarnation
(
   inx IN NUMBER,
   pd IN NUMBER,
   inc OUT NUMBER
)
IS
BEGIN
   SELECT Incarnation INTO inc FROM OF_Services
      WHERE ServiceTypeIndex = inx AND PID = pd;
EXCEPTION
      WHEN NO_DATA_FOUND
   THEN
      inc := -1;
END of_incarnation;
/

CREATE PROCEDURE of_max_incarnation
(
   pd IN NUMBER,
   inc OUT NUMBER
)
IS
BEGIN
   SELECT MAX (Incarnation) INTO inc FROM OF_Services
      WHERE  PID = pd;
END of_max_incarnation;
/

CREATE PROCEDURE of_describe_services
(
   inc IN NUMBER,
   pd IN NUMBER,
   msk OUT NUMBER,
   int OUT NUMBER
)
IS
BEGIN
   SELECT Masked, InterfaceTypeIndex INTO msk, int
      FROM OF_Services
      WHERE Incarnation = inc AND PID = pd;
END of_describe_services;
/

CREATE PROCEDURE of_update_mask
(
   inc IN NUMBER,
   pd IN NUMBER,
   mk IN NUMBER,
   found OUT NUMBER
)
IS
BEGIN
   UPDATE OF_Services
      SET Masked = mk
      WHERE Incarnation = inc AND PID = pd;
   IF SQL%FOUND THEN
      found := 1;
   ELSE
      found := 0;
   END IF;
END of_update_mask;
/

CREATE PROCEDURE of_insert_services
(
   inc IN NUMBER,
   pd IN NUMBER,
   idx IN NUMBER,
   mk IN NUMBER,
   int IN NUMBER
)
IS
BEGIN
   INSERT INTO OF_Services
      (Incarnation, PID, ServiceTypeIndex, Masked, InterfaceTypeIndex)
      VALUES (inc, pd, idx, mk, int);
END of_insert_services;
/

CREATE PROCEDURE of_insert_supertypes
(
   inc IN NUMBER,
   pd IN NUMBER,
   st IN NUMBER
)
IS
BEGIN
   INSERT INTO OF_SuperTypes
      (Incarnation, PID, ServiceType)
      VALUES (inc, pd, st);
END of_insert_supertypes;
/

CREATE PROCEDURE of_delete_services
(
   inc IN NUMBER,
   pd IN NUMBER
)
IS
BEGIN
   DELETE FROM OF_Services
      WHERE Incarnation = inc AND PID = pd;
END of_delete_services;
/

CREATE PROCEDURE of_delete_service_all
(
  inc IN NUMBER,
  pd IN NUMBER
)
IS
BEGIN
   DELETE FROM OF_Services WHERE Incarnation = inc AND PID = pd;
   DELETE FROM OF_ServiceProperties WHERE Incarnation = inc AND PID = pd;
   DELETE FROM OF_SuperTypes WHERE Incarnation = inc AND PID = pd;
END of_delete_service_all;
/

CREATE PROCEDURE of_delete_sproperties
(
   inc IN NUMBER,
   pd IN NUMBER
)
IS
BEGIN
   DELETE FROM OF_ServiceProperties
      WHERE Incarnation = inc AND PID = pd;
END of_delete_sproperties;
/

CREATE PROCEDURE of_delete_supertypes
(
   inc IN NUMBER,
   pd IN NUMBER
)
IS
BEGIN
   DELETE FROM OF_SuperTypes
      WHERE Incarnation = inc AND PID = pd;
END of_delete_supertypes;
/


CREATE PROCEDURE of_delete_offer
(
   oid IN VARCHAR2,
   pd IN NUMBER,
   px IN NUMBER,
   res OUT NUMBER
)
IS
  kn NUMBER;
  isp NUMBER;
BEGIN
   SELECT COUNT(*) INTO kn FROM OF_Offers WHERE OfferID = oid AND PID = pd;
   IF kn = 0 THEN
      res := 2;
   ELSE
      SELECT IsProxy INTO isp FROM OF_Offers WHERE OfferID = oid AND PID = pd;
      IF isp = px THEN
         res := 0;
         DELETE FROM OF_Offers WHERE OfferID = oid AND PID = pd;
      ELSE
         res := 1;
      END IF;
   END IF;
END of_delete_offer;
/

CREATE INDEX of_offerPropsIndexValue ON OF_OfferProps(Value, Name, ServiceTypeIndex, PID)
;

CREATE INDEX of_offerPropsIndexNumValue ON OF_OfferProps(NumVal, Name, ServiceTypeIndex, PID)
;

CREATE INDEX of_offerPropsIndexComplex ON OF_OfferProps(Name, ComplexValue)
;

CREATE INDEX of_offersIndex ON OF_Offers (ServiceTypeIndex, PID, MatchAll)
;
