/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNotification.examples.Benchmark;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;
import org.omg.CosNotifyFilter.*;

import com.prismt.orb.ObjectAdapter;


/**
 * A collection of utility methods for the benchmark clients.
 */
public class BenchmarkUtils
{
   /** Numeric codes for extractBooleanArg(). */
   public static final int NO_VALUE_SPECIFIED         = 0;
   public static final int TRUE_SPECIFIED             = 1;
   public static final int FALSE_SPECIFIED            = 2;
   public static final int UNEXPECTED_VALUE_SPECIFIED = 3;


  /**
    * Extracts a string value for a specific switch from an array of arguments.
    * @param args   The argument array (e.g. command-line).
    * @param arg    The argument to search for.
    * @param def    The default value to use, if the argument is not found or
    *               if there is no argument following it.
    * @return       The found argument or the default.
    */
   public static synchronized String extractArg (String[] args, String arg, String def)
   {
      String ret = def;

      for (int i = 0 ; i < args.length ; i++)
      {
         if (args [i].equals (arg))
         {
            if (i <= (args.length-2))
            {
               ret = args [i+1];
            }
            break;
         }
      }

      return ret;
   }


  /**
    * Extracts a boolean value for a specific switch from an array of arguments.
    * @param args   The argument array (e.g. command-line).
    * @param arg    The argument to search for.
    * @param def    The default value to use, if the argument is not found or
    *               if there is no argument following it.
    * @return       The found argument or the default.
    */
   public static synchronized boolean extractArg (String[] args, String arg, boolean def)
   {
      String ret = extractArg (args, arg, (def ? "true" : "false"));
      return (ret.equalsIgnoreCase ("true"));
   }


  /**
    * A variant method to extract a boolean argument that does not use a default
    * value and allows the caller to determine if the switch wasn't specified.
    * @param args   The argument array (e.g. command-line).
    * @param arg    The argument to search for.
    * @return       A numeric code.
    */
   public static synchronized int extractBooleanArg (String[] args, String arg)
   {
      String noValSpec = "<NO_VALUE_SPECIFIED>";
      String ret = extractArg (args, arg, noValSpec);

      if (ret.equalsIgnoreCase (noValSpec))
      {
         return NO_VALUE_SPECIFIED;
      }

      if (ret.equalsIgnoreCase ("true"))
      {
         return TRUE_SPECIFIED;
      }
      
      if (ret.equalsIgnoreCase ("false"))
      {
         return FALSE_SPECIFIED;
      }

      return UNEXPECTED_VALUE_SPECIFIED;
   }


  /**
    * Extracts a numeric value for a specific switch from an array of arguments.
    * @param args   The argument array (e.g. command-line).
    * @param arg    The argument to search for.
    * @param def    The default value to use, if the argument is not found or
    *               if there is no argument following it.
    * @return       The found argument or the default.
    */
   public static synchronized int extractArg (String[] args, String arg, int def)
   {
      int retVal = 0;

      try
      {
         String ret = extractArg (args, arg, Integer.toString (def));
         retVal = Integer.parseInt (ret);
      }
      catch (NumberFormatException nfex)
      {
         retVal = def;
      }

      return retVal;
   }


  /**
    * Extracts a numeric value for a specific switch from an array of arguments.
    * @param args   The argument array (e.g. command-line).
    * @param arg    The argument to search for.
    * @param def    The default value to use, if the argument is not found or
    *               if there is no argument following it.
    * @return       The found argument or the default.
    */
   public static synchronized long extractArg (String[] args, String arg, long def)
   {
      long retVal = 0;

      try
      {
         String ret = extractArg (args, arg, Long.toString (def));
         retVal = Long.parseLong (ret);
      }
      catch (NumberFormatException nfex)
      {
         retVal = def;
      }

      return retVal;
   }


   /**
    * Returns a reference to the ORB.
    * @param args    Arguments for initialisation (i.e. the command line arguments).
    * @param caller  Caller object (used in the output).
    * @return      A reference to the ORB.
    * @throws Exception
    */
   public static synchronized org.omg.CORBA.ORB getORB (String[] args, Object caller)
         throws Exception
   {
      System.out.println (caller + ": getting ORB reference...");
      org.omg.CORBA.ORB orb = ObjectAdapter.init (args);
      if (orb == null)
      {
         throw new Exception ("Returned ORB reference was null");
      }

      return orb;
   }


   /**
    * Returns a reference to the event channel factory.
    * @param orb          A reference to the ORB instance.
    * @param resolveName  The resolve name to use for the notification service.
    * @param caller       Caller object (used in the output).
    * @return             A reference to the event channel factory.
    * @throws Exception
    */
   public static synchronized EventChannelFactory getEventChannelFactory
                                (org.omg.CORBA.ORB orb, String resolveName,
                                 Object caller)
         throws Exception
   {
      System.out.println (caller + ": resolving to notification service using " +
                          resolveName + "...");
      org.omg.CORBA.Object cobj = orb.resolve_initial_references (resolveName);
      EventChannelFactory factory = EventChannelFactoryHelper.narrow (cobj);
      if (factory == null)
      {
         throw new Exception ("Returned event channel factory reference was null");
      }

      return factory;
   }


   /**
    * Returns a reference to an event channel factory.  If the channel with the
    * specified ID cannot be found, a new channel is created with the specific
    * mode.
    * @param orb         A reference to the ORB instance.
    * @param factory     A reference to the event channel factory instance.
    * @param id          The ID of the channel to find.
    * @param caller      Caller object (used in the output).
    * @return            A reference to the event channel.
    * @throws Exception  If the channel could not be found, or if a general
    *                    error occurred.
    */
   public static synchronized EventChannel getEventChannel (org.omg.CORBA.ORB orb,
                                                            EventChannelFactory factory,
                                                            int id,
                                                            Object caller)
         throws Exception
   {
      EventChannel channel = null;

      System.out.println (caller + ": trying to find channel with ID " + id + "...");
      try
      {
         channel = factory.get_event_channel (id);
         System.out.println (caller + ": found channel with ID " + id);
      }
      catch (ChannelNotFound ex)
      {
         /* Channel 0 was not found so create a channel with no QoS
          * or admin properties.
          */
         System.out.println (caller + ": unable to find channel so creating one");

          /* Admin property, id holder and QoS property variables.*/
          Property[] qosProp = new Property[0];
          Property[] admProp = new Property[0];
          org.omg.CORBA.IntHolder myid = new org.omg.CORBA.IntHolder ();

          try
          {
             /* Create a new channel */
             channel = factory.create_channel (qosProp, admProp, myid);
             System.out.println (caller + ": created channel");
          }
          catch (UnsupportedQoS exc)
          {
             /* Thrown if unknown QoS properties were specified */
             System.err.println ("Failed to create channel: unsupported QoS properties specified!");
             exc.printStackTrace ();
             System.exit (-1);
          }
          catch (UnsupportedAdmin exc)
          {
             /* Thrown if we specified unknown admin properties */
             System.err.println ("Failed to create channel: unsupported admin properties specified");
             exc.printStackTrace ();
             System.exit (-1);
          }
       }
      
      
      return channel;
   }
}


// EOF
