/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/
/**
 * @author PrismTech
 * @version Version 2.5
 */

package com.prismt.cos.CosNotification.examples.Filter;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;
import org.omg.CosNotifyFilter.*;

import com.prismt.orb.*;

/**
 * Example Push Consumer. <code>PushConsumer</code>This shows how to create a basic push consumer
 * that connects to channel 0, if it exists. If the channel does not exist it
 * is created and the consumer will receive structured events from this event
 * channel. The consumer uses a custom filter attached to the proxy so that it
 * will only recieve events which have a length property between 20 and
 * 40 and a height property between 10 and 20.
 */
public class PushConsumer implements StructuredPushConsumerOperations
{
   /* The ORB */
   private static org.omg.CORBA.ORB orb;
   /* The event channel */
   private EventChannel channel = null;
   /* The resolveName of the NotificationService */
   private String resolveName = "NotificationService";

   /**
    * The <code>main</code> method. This initialises the ORB and constructs
    * a Push Consumer object. The push consumer is then connected to the proxy
    * and waits for events.
    * @param args a <code>String[]</code> Arguments passed in
    */
   public static void main (String [] args)
   {
      /* Initialise the ORB */
      try
      {
         orb = ObjectAdapter.init (args);
      }
      catch( Exception e )
      {
         e.printStackTrace();
         System.exit(1);
      }

      /* Create a new instance of a Push Consumer */
      System.out.println("Creating PushConsumer");
      PushConsumer pc = new PushConsumer ();

      /* Connect to the proxy push supplier using the connect method.*/
      System.out.println ("Connecting to proxy");
      pc.connect ();
      System.out.println("Ready for events...");
      /* Ready the Object Adapter */
      ObjectAdapter.ready ();
   }

   /**
    * The constuctor for the class which creates a new
    * <code>PushConsumer</code> instance.
    */
   public PushConsumer ()
   {
      /* CORBA Object reference to the Notification Service.*/
      org.omg.CORBA.Object obj = null;
        /* Event channel factory */
      EventChannelFactory factory = null;

      /* create a CORBA object for this instance of the PushConsumer*/
      ObjectAdapter.createTransient (this);

      try
      {
        /* In order to avoid problems with VisiBroker when registering the
         name NotificationService, rename it */

        if ((com.prismt.openfusion.Version.getORB().toUpperCase().startsWith("VISIBROKER-52")) ||
           (com.prismt.openfusion.Version.getORB().toUpperCase().startsWith("VISIBROKER-60")))
        {
           resolveName = "OFNotificationService";
        }
     
        /* End of specific code to deal with VisiBroker and registering
         NotificationService name problem*/

        /* resolve the Notification Service */
        System.out.println
           ("Resolving initial references to "+ resolveName);
         obj = orb.resolve_initial_references (resolveName);
         System.out.println ("Success");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         /* Thrown if there is no Notification Service registered with the
          * ORB or if the call to resolve_initial_references returns null.
          */
         System.err.println ("Failed to resolve Notification Service");
         System.exit (1);
      }

      /* obtain an object reference to the EventChannelFactory */
      factory = EventChannelFactoryHelper.narrow (obj);

      try
      {
         /* try and obtain the event channel with ID 0  from the factory*/
         System.out.println ("Trying to get channel 0 ...");
         channel = factory.get_event_channel (0);
         System.out.println ("Success");
      }
      catch (ChannelNotFound ex)
      {
          /*
          * thrown if the channel with ID 0 does not exist. A new channel
          * is created which has no Admin or QoS properties set.
          */
         System.out.println
            ("Failed, channel not found. Creating new channel ...");

         /* Admin property, id holder and QoS property variables.*/
         Property[] qosProp = new Property [0];
         Property[] admProp = new Property [0];
         org.omg.CORBA.IntHolder id = new org.omg.CORBA.IntHolder ();

         try
         {
            /* Create a new channel */
            channel = factory.create_channel (qosProp, admProp, id);
         }
         catch (UnsupportedQoS exc)
         {
            /* Thrown if unknown QoS properties were specified */
            System.out.println ("Unsupported QoS properties specified!");
         }
         catch (UnsupportedAdmin exc)
         {
            /* Thrown if we specified unknown admin properties */
            System.out.println ("Unsupported admin properties specified");
         }
      }
   }

   /**
    * The <code>connect</code> method is used to obtain a proxy push supplier
    * from the admin object and connect the push consumer to it.
    * @return a <code>StructuredProxyPushSupplier</code> instance
    */
   public StructuredProxyPushSupplier connect ()
   {
      /* Proxy push supplier variable */
      StructuredProxyPushSupplier sp;
      /* Defines the type of proxy required */
      ClientType ctype = ClientType.STRUCTURED_EVENT;
      /* Holder to hold the proxy id */
      org.omg.CORBA.IntHolder pid = new org.omg.CORBA.IntHolder ();
      /* Proxy supplier variable */
      ProxySupplier proxy = null;
      /* The filter varible */
      Filter filter = null;
      /* The classname of the filter being used */
      String classname = "com.prismt.cos.CosNotification.examples.Filter.MyFilter";

      /* Obtain the filter factory from the event channel */
      FilterFactory factory = channel.default_filter_factory ();
      /* Obtain the consumer admin object reference*/
      ConsumerAdmin admin = channel.default_consumer_admin ();

      try
      {
         /* Create the filter */
         System.out.println("Creating filter");
         filter = factory.create_filter (classname);
      }
      catch (InvalidGrammar ex)
      {
         /*
          *This exception is thrown if the filter class can not be
          * found. The classname needs to be on the CLASSPATH of the
          * console window where the Notification Service is being run
          */
         System.err.println("Filter " + classname + "not found");
      }

      try
      {
         /* obtain a structured push supplier object reference.*/
         proxy = admin.obtain_notification_push_supplier (ctype, pid);
         /* Add a filter to the proxy */
         System.out.println("Adding filter");
         proxy.add_filter (filter);
      }
      catch (AdminLimitExceeded ex)
      {
         /* Thrown if the admin object is unable to have any more proxy
          * suppliers associated with it.
          */
         System.err.println ("Maximum number of proxies exceeded");
         System.exit (1);
      }

      /* Narrow the proxy supplier to a Structured Proxy Push Supplier */
      sp = StructuredProxyPushSupplierHelper.narrow (proxy);

      try
      {
         /* connect the consumer to the proxy */
         sp.connect_structured_push_consumer
            (StructuredPushConsumerHelper.narrow
             (ObjectAdapter.getObject (this)));
      }
      catch (org.omg.CosEventChannelAdmin.TypeError ex)
      {
         /*
          * This exception is thrown if you attempt to connect
          * a sequenced consumer to a structured proxy or vice versa.
          */
         System.err.println ("Type Error");
         System.exit (1);
      }
      catch (org.omg.CosEventChannelAdmin.AlreadyConnected ex)
      {

          /*
          * This exception is thrown if a consumer is already connected
          * to this proxy. This should not be thrown because the proxy
          * has just been created.
          */
         System.err.println ("Already connected");
         System.exit (1);
      }
      return sp;
   }

   /**
    * The <code>disconnect_structured_push_consumer</code> method is
    * defined in the StructuredPushConsumerOperations interface. It allows
    * the event channel to disconnect the push consumer from the channel.
    */
   public void disconnect_structured_push_consumer ()
   {
      System.out.println ("Disconnected");
      System.exit (0);
   }

   /**
    * The <code>offer_change</code> method is defined in the
    * StructuredPushConsumerOperations interface. It is called by the
    * channel to inform the consumer of new event types available or those no
    * longer offered
    * @param added[] the new <code>EventType</code> offered
    * @param removed[] the <code>EventType</code> that was removed
    */
   public void offer_change (EventType added[], EventType removed[])
   {
   }

   /**
    * This method is defined in the StructuredPushConsumerOperations
    * interface. It is a callback method that is called by the
    * NotificationService when an event is received by the channel.
    * The method attempts to extract a long from the event received
    * and prints this out to the screen.
    * @param event the <code>StructuredEvent</code> to be sent
    */
   public void push_structured_event (StructuredEvent event)
   {
      org.omg.CORBA.Any data = event.remainder_of_body;
      int value = data.extract_long ();
      System.out.println
      (
         "Received event: "
         + value
         + " Length "
         +  event.filterable_data[0].value.extract_long ()
         + " Height "
         +  event.filterable_data[1].value.extract_long ()
      );
   }
}
