/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 2.5
 */

package com.prismt.cos.CosNotification.examples.Filter;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;

import com.prismt.orb.*;

import java.util.Random;

/*
 * Example Push Supplier. This shows how to create a simple push
 * supplier that creates structured events and sends them  to the event
 * channel. It will connect to channel 0, if it exists. If it does not exist
 * it will create the channel and push the structured events to it.
 */
public class PushSupplier implements StructuredPushSupplierOperations
{
   /* The ORB*/
   private static org.omg.CORBA.ORB orb;
   /* The event channel */
   private EventChannel channel = null;
   /* A counter to enable the events to be numbered */
   private int counter = 0;
   /* Allows random numbers to be generated */
   Random rand;
   /* The resolveName of the NotificationService */
   private String resolveName = "NotificationService";

   /**
    *  The <code>main</code> method. This initialises the ORB and constructs
    * a Push Supplier object. The push supplier is then connected to the proxy
    * and sends events.
    *
    * @param args a <code>String[]</code> Arguments passed in.
    */
   public static void main (String[] args)
   {
      /* Initialise the ORB */
      try
      {
         orb = ObjectAdapter.init(args);
      }
      catch( Exception e )
      {
         e.printStackTrace();
         System.exit(1);
      }

      /* Create a new instance of a push supplier */
      System.out.println ("Creating a new push supplier instance");
      final PushSupplier ps = new PushSupplier ();
      /* Connect to the proxy push consumer using the connect method */
      System.out.println ("Connecting to proxy");
      final StructuredProxyPushConsumer proxy = ps.connect ();

      /* push structured events */
      System.out.println ("Pushing events ... ");
      Thread thread = new Thread ()
      {
         public void run ()
         {
            ps.pushEvents (proxy);
         }
      };
      thread.start ();

      /* ready the ObjectAdapter */
      ObjectAdapter.ready ();
   }

   public PushSupplier ()
   {
      org.omg.CORBA.Object obj = null;
      EventChannelFactory factory = null;

      /* create a CORBA object for this instance of the PushSupplier */
      ObjectAdapter.createTransient (this);

      try
      {
        /* In order to avoid problems with VisiBroker when registering the
         name NotificationService, rename it */

        if ((com.prismt.openfusion.Version.getORB().toUpperCase().startsWith("VISIBROKER-52")) ||
           (com.prismt.openfusion.Version.getORB().toUpperCase().startsWith("VISIBROKER-60")))
        {
           resolveName = "OFNotificationService";
        }
        
        /* End of specific code to deal with VisiBroker and registering
         NotificationService name problem*/

        /* resolve the Notification Service */
        System.out.println
           ("Resolving initial references to "+ resolveName);
         obj = orb.resolve_initial_references (resolveName);
         System.out.println ("Success");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println("Failed to resolve Notification Service");
         System.exit (1);
      }

      /* obtain an object reference to the EventChannelFactory */
      factory = EventChannelFactoryHelper.narrow (obj);

      try
      {
         System.out.println ("Looking for channel 0");
         /* try and obtain the event channel with ID 0 */
         channel = factory.get_event_channel (0);
         System.out.println ("Channel found");
      }
      catch (ChannelNotFound ex)
      {

         /* Channel 0 was not found so create a channel with no QoS
          * or admin properties.
          */
         System.out.println ("Unable to find channel so creating one");

         /* Admin property, id holder and QoS property variables.*/
         Property[] qosProp = new Property[0];
         Property[] admProp = new Property[0];
         org.omg.CORBA.IntHolder id = new org.omg.CORBA.IntHolder ();

         try
         {
            channel =  factory.create_channel (qosProp, admProp, id);
         }
         catch (UnsupportedQoS exc)
         {
             /* Thrown if unknown QoS properties were specified */
            System.out.println ("Unsupported QoS properties specified!");
         }
         catch (UnsupportedAdmin exc)
         {
             /* Thrown if we specified unknown admin properties */
            System.out.println ("Unsupported admin properties specified");
         }
      }
   }

   /**
    * Connects to the proxy structured push consumer
    * @return a <code>StructuredProxyPushConsumer</code> instance
    */
   public StructuredProxyPushConsumer connect ()
   {
      /* Proxy push consumer variable */
      StructuredProxyPushConsumer sp;
      /* Defines the type of proxy required */

      ClientType ctype = ClientType.STRUCTURED_EVENT;
      /* Holder to hold the proxy id */
      org.omg.CORBA.IntHolder pid = new org.omg.CORBA.IntHolder ();
      ProxyConsumer proxy = null;
      /* Obtain the supplier admin object reference */
      SupplierAdmin admin = channel.default_supplier_admin ();

      try
      {
         /* obtain a structured push consumer object reference */
         proxy =  admin.obtain_notification_push_consumer (ctype, pid);
      }
      catch (AdminLimitExceeded ex)
      {

         /* Thrown if the admin object is unable to have any more proxy
          * consumers associated with it.
          */
         System.err.println ("Maximum number of proxies exceeded");
         System.exit (1);
      }

      /* Narrow the proxy consumer to a Structured Proxy Push consumer */
      sp = StructuredProxyPushConsumerHelper.narrow (proxy);

      try
      {
         /* connect to the proxy */
         sp.connect_structured_push_supplier
         (StructuredPushSupplierHelper.narrow
             (ObjectAdapter.getObject (this)));
      }
      catch (org.omg.CosEventChannelAdmin.AlreadyConnected ex)
      {
         /* Thrown if the proxy already has a connected supplier*/
         System.err.println ("Already Connected");
         System.exit (1);
      }
      return sp;
   }

    /**
   * The <code>disconnect_sequence_push_supplier</code> method is used
   * by the event channel to disconnect the supplier.
   */
   public void disconnect_structured_push_supplier ()
   {
      System.out.println ("disconnected");
      System.exit (0);
   }

   /**
    * The <code>subscription_change</code> method allows the event channel to
    * notify the supplier of the types of events required by the consumers.
    * @param added[] the <code>EventType</code> now required
    * @param removed[] the <code>EventType</code> that has been removed
    */
   public void subscription_change (EventType added[], EventType removed[])
   {
   }


   /** Push structured events
    * @param StructuredProxyPushConusmer the proxy object
    */
   void pushEvents (StructuredProxyPushConsumer proxy)
   {
      //infinite loop for pushing events onto the proxy
      while (true)
      {
         rand = new Random ();
         int randLength  = Math.abs (rand.nextInt (60));
         int randHeight =  Math.abs (rand.nextInt (60));

         try
         {
             /*
              * create structured events which contain two properties in
              * the filterable data field. The height property can be between
              * 0 and 60 and the length property can be between 0 and 60
              * */
            StructuredEvent event = new StructuredEvent ();

            EventType type = new EventType ("example", "test");
            FixedEventHeader fixed = new FixedEventHeader (type, "event");

            Property variable[] = new Property [0];

            org.omg.CORBA.Any data = orb.create_any ();
            data.insert_long (counter ++);

            event.header = new EventHeader (fixed, variable);
            event.filterable_data = new Property [2];
            event.filterable_data[0] = new Property ();
            event.filterable_data[0].name = "length";
            event.filterable_data[0].value = orb.create_any ();
            event.filterable_data[0].value.insert_long (randLength);
            event.filterable_data[1] = new Property ();
            event.filterable_data[1].name = "height";
            event.filterable_data[1].value = orb.create_any ();
            event.filterable_data[1].value.insert_long (randHeight);
            event.remainder_of_body = data;

            proxy.push_structured_event (event);
            System.out.println
            (
               "Pushed event: "
               + (counter-1)
               + " Length "
               +  event.filterable_data[0].value.extract_long ()
               + " Height "
               +  event.filterable_data[1].value.extract_long ()
            );
            /* Wait for a second before sending the next event */
            Thread.sleep (1000);
         }
         catch (InterruptedException ex) {}
         catch (org.omg.CosEventComm.Disconnected ex)
         {
            System.out.println ("Proxy was disconnected");
            System.exit (0);
         }
      }
   }
}
