package com.prismt.cos.CosNotification.examples.Persistence;

/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 2.5
 */

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;
import org.omg.CosNotifyFilter.*;


import com.prismt.util.PID;
import com.prismt.util.UUID;
import com.prismt.orb.ObjectAdapter;
import com.prismt.cos.CosNotification.NotificationExtensions.AutoSequenceBatchSize;

import java.io.*;

/**
 * Example Persistent Consumer. This shows how to create a basic persistent
 * consumer for the Notification Service. The first time the code is run,
 * a normal Persistent Consumer will be created which is connected to a proxy
 * using an event channel with id 0. The consumer is able to receive events
 * from any supplier connected to channel 0. The consumer can be terminated by
 * pressing Ctrl-C. If the program is run again, it realises that there is
 * a stale reference to the old consumer and will re-activate itself and
 * replace the old consumer. The newly created consumer will then be able to
 * receive events again, and it will also receive any events that were pushed
 * onto the channel whilst it was down.
 */
public class PersistentConsumer
   implements StructuredPushConsumerOperations, java.io.Serializable
{
   /**
    * The resolve name for the notification service.  If the user does not
    * specify this via the "-resolve" parameter the default is used.
    */
   private static String resolveName = null;
   private static String defaultResolveName = "NotificationService";


   /**
    * String used to hold the IOR of the old consumer (if there is an old
    * consumer). 
    */
   private String consumerRef = "";


   /**
    * The reference to the ORB instance.
    */
   private transient org.omg.CORBA.ORB orb;


   /**
    * Extracts a value for a specific switch from an array of arguments.
    * @param args   The argument array (e.g. command-line).
    * @param arg    The argument to search for.
    * @param def    The default value to use, if the argument is not found or
    *               if there is no argument following it.
    * @return       The found argument or the default.
    */
   private static String extractArg (String[] args, String arg, String def)
   {
      String ret = def;

      for (int i = 0 ; i < args.length ; i++)
      {
         if (args[i].equals(arg))
         {
            if (i <= (args.length-2))
            {
               ret = args[i+1];
            }
            break;
         }
      }

      return ret;
   }


   /**
    * The only constructor for this class.
    * @param orb   An initialised ORB refererence.
    */
   public PersistentConsumer(org.omg.CORBA.ORB orb)
   {
      this.orb = orb;
   }


   /**
    * Method to get event channel 0 or create a new channel.
    * @return an <code>EventChannel</code> instance.
    */
   private EventChannel getChannel()
   {
      /* CORBA Object reference to the notification service. */
      org.omg.CORBA.Object notRef = null;

      /* Reference to the event Channel factory. */
      EventChannelFactory factory = null;

      /* The event channel to return. */
      EventChannel channel = null;

      try
      {
         /* Resolve the Notification Service. */
         System.out.println("Resolving to the notification service using " +
                            resolveName);
         notRef = orb.resolve_initial_references(resolveName);
         System.out.println("Resolve success");
         if(notRef == null)
         {
            System.out.println("Received a null reference when resolving to " +
                               "the notification service.");
            System.exit(1);
         }
      }
      catch(org.omg.CORBA.ORBPackage.InvalidName in)
      {
         /*
          * Thrown if there is no notification service registered with the orb.
          */
         System.out.println("Could not resolve to the notification service.");
         in.printStackTrace();
         System.exit(1);
      }

      /* Narrow the reference to the event channel factory. */
      factory = EventChannelFactoryHelper.narrow(notRef);
      
      /* Admin property, id holder and QoS property variables.*/
      Property[] qosProps = new Property[2];
      Property[] admProp = new Property[0];
      org.omg.CORBA.IntHolder id = new org.omg.CORBA.IntHolder ();

      /* Set the qos properties to have persistent connections and events */
      qosProps[0] = new Property();
      qosProps[0].name = ConnectionReliability.value;
      qosProps[0].value = orb.create_any();
      qosProps[0].value.insert_short(Persistent.value);

      qosProps[1] = new Property();
      qosProps[1].name = EventReliability.value;
      qosProps[1].value = orb.create_any();
      qosProps[1].value.insert_short(Persistent.value);

      try
      {
         /* Try to get event channel 0 from the factory. */
         System.out.print("Trying to get channel 0...");
         channel = factory.get_event_channel(0);
         System.out.println(" success");
         
         channel.set_qos (qosProps);
         System.out.println ("Set qos on channel");
      }
      catch (UnsupportedQoS uqos)
      {
         /* Thrown if we specified unknown qos properties. */
         System.out.println("Unsupported qos properties specified!");
         uqos.printStackTrace();
         System.exit(1);  
      }
      catch(ChannelNotFound cnf)
      {
         /* Thrown if channel with id 0 does not exist. */
         System.out.println(" failed: channel not found");
         System.out.println("Creating new channel...");

         /* QoS and Admin properties and ID holder. */
         org.omg.CORBA.IntHolder channelId = new org.omg.CORBA.IntHolder();

         Property[] adminProps = new Property[0];

         try
         {
            /* Create the channel with the required properties. */
            channel = factory.create_channel(qosProps, adminProps, channelId);
            System.out.println("Created channel with ID " + channelId.value);
         }
         catch(UnsupportedQoS uqos)
         {
            /* Thrown if we specified unknown qos properties. */
            System.out.println("Unsupported qos properties specified!");
            uqos.printStackTrace();
            System.exit(1);
         }
         catch(UnsupportedAdmin ua)
         {
            /* Thrown if we specified unknown admin properties. */
            System.out.println("Unsupported admin properties specified!");
            ua.printStackTrace();
            System.exit(1);
         }
      }

      /* Return the channel we just found/created. */
      return channel;
   }


   /**
    * Initialisation method; determines if there has been a previous instance
    * of the consumer.  If there has been, then it set the "create" flag to be
    * false.  Otherwise the create flag is set to true.
    * 
    * If there has been a previous instance of a conumer, the Process.PID
    * system property is set, so this instance of the consumer will have the
    * same ID as the old instance when the orb is initialised.
    * 
    * param consRef   The old consumer reference: <code>null</code> if there
    *                 was no old consumer.
    */
   public void init(String consRef)
   {
      EventChannel channel;

      this.consumerRef = consRef;

      /* Get channel 0 (or create a new channel). */
      channel = getChannel();

      /* Create a new consumer if required. */
      if (this.consumerRef == null)
      {
         /* Create a new consumer. */
         System.out.println("Creating a new consumer");
         createNewConsumer(channel);
      }
      else
      {
         /* Otherwise we need to re-activate the old consumer. */
         System.out.println("Reactivating the old consumer");
         useOld();
      }
   }


   /**
    * Method to create a new consumer.  This retrieves the default consumer
    * admin from the given channel. obtains a proxy from the admin, sets the
    * QoS properties on the proxy and connects the consumer to the proxy.  The
    * details of the consumer are written to a file called "pid".
    * 
    * @param channel   The <code>EventChannel</code> to use.
    */
   private void createNewConsumer(EventChannel channel)
   {
      /* Get the default consumer admin. */
      ConsumerAdmin ca = channel.default_consumer_admin();

      /* Get a proxy supplier from the admin. */
      ProxySupplier ps = null;
      try
      {
         org.omg.CORBA.IntHolder idHolder = new org.omg.CORBA.IntHolder();
         ps = ca.obtain_notification_push_supplier
            (ClientType.STRUCTURED_EVENT, idHolder);
         System.out.println("Obtained a proxy supplier with ID " +
                            idHolder.value);
      }
      catch(AdminLimitExceeded ali)
      {
         /*
          * Thrown if the admin is unable to have any more ProxySuppliers
          * associated with it.
          */
         System.out.println("Admin limit was exceeded when obtaining " +
                            "a proxy supplier");
         ali.printStackTrace();
         System.exit(1);
      }

      /* Narrow the proxy supplier to a StructuredProxyPushSupplier. */
      StructuredProxyPushSupplier proxy =
      StructuredProxyPushSupplierHelper.narrow(ps);

      /* Set up the required QoS for the proxy supplier. */
      /* Turn batching off for persistent messaging. */
      Property[] qosProps = new Property[2];
      qosProps[0] = new Property();
      qosProps[0].name = ConnectionReliability.value;
      qosProps[0].value = orb.create_any();
      qosProps[0].value.insert_short(Persistent.value);
      qosProps[1] = new Property ();
      qosProps[1].name = AutoSequenceBatchSize.value;
      qosProps[1].value = orb.create_any ();
      qosProps[1].value.insert_long (0);
      try
      {
         proxy.set_qos(qosProps);
         System.out.println("QOS properties set on proxy");
      }
      catch(UnsupportedQoS uqos)
      {
         /* Thrown if we specified unknown QoS properties. */
         System.out.println("Unsupported QoS properties specified!");
         uqos.printStackTrace();
         System.exit(1);
      }

      /* Create "this" class as a CORBA object. */
      org.omg.CORBA.Object thisCorbaObject =
         ObjectAdapter.createPersistent(this);

      /* Narrow the created corba object to a StructuredPushConsumer. */
      StructuredPushConsumer consumer =
         StructuredPushConsumerHelper.narrow(thisCorbaObject);

      /* Connect the consumer to the proxy. */
      try
      {
         proxy.connect_structured_push_consumer(consumer);
      }
      catch(org.omg.CosEventChannelAdmin.AlreadyConnected ac)
      {
         System.out.println("AlreadyConnected exception was thrown when " +
                            "connecting the proxy and consumer.");
         ac.printStackTrace();
         System.exit(1);
      }
      catch(Exception e)
      {
         System.out.println("An exception was thrown when connecting the " +
                            "proxy and consumer.");
         e.printStackTrace();
         System.exit(1);
      }

      /* Write the consumer's information to a file called "pid". */
      try
      {
         /* Construct a PrintWriter. */
         PrintWriter pw = new PrintWriter(new FileOutputStream("pid"));

         /* Write and print to the screen the PID of the consumer. */
         pw.println(PID.getPID());
         System.out.println("Writing out PID: " + PID.getPID());

         /* Write and print to the screen the consumer's IOR. */
         String theIOR = orb.object_to_string(consumer);
         pw.println(theIOR);
         System.out.println("Writing out consumer reference: " + theIOR);

         /* Flush and close the print writer.*/
         pw.flush();
         pw.close();
      }
      catch(IOException ioe)
      {
         System.out.println ("An IOException was thrown when writing out the "+
                             "consumer information.");
         ioe.printStackTrace();
         System.exit(1);
      }
   }


   /**
    * Method used to re-activate the old consumer's CORBA object.  The method
    * narrows the reference to the old consumer, and then reactivates it.
    */
   private void useOld()
   {
      /*
       * Narrow the old consumer reference.  The reference is obtained in the
       * main() method, and is passed to the init() method which stores it in the
       * class member.
       */
      StructuredPushConsumer oldConsumer = StructuredPushConsumerHelper.narrow
         (orb.string_to_object(this.consumerRef));
      System.out.println("Old consumer reference: " + oldConsumer);

      /* Reactivate the old consumer. */
      java.io.Serializable obj = ObjectAdapter.reactivate(oldConsumer);
      System.out.println("Old consumer reactivated: " + obj);
   }


   /**
    * This method is defined in the StructuredPushConsumerOperations interface.
    */
   public void disconnect_structured_push_consumer()
   {
   }


   /**
    * This method is defined in the StructuredPushConsumerOperations interface.
    * It is called by the channel to inform the consumer of new event types
    * available or those no longer offered.
    * @param added the new <code>EventType[]</code> offered
    * @param removed the <code>EventType[]</code> that was removed.
    */
   public void offer_change(EventType[] added, EventType[] removed)
   {
   }


   /**
    * This method is defined in the StructuredPushConsumerOperations interface.
    * It is a callback method that is called by the notification service when
    * an event is received by the channel.  The method  prints the event out
    * to the screen.
    * @param event   The <code>StructuredEvent</code> sent.
    */
   public void push_structured_event(StructuredEvent event)
   {
      try
      {
         int data = event.remainder_of_body.extract_long();
         System.out.println("Received Event: " + data);
      }
      catch(Exception e)
      {
         System.out.println("Received Event: could not extract data");
      }
   }


   /**
    * Main method.  This constructs and initialises the PersistentConusmer
    * instance.  It checks if there has been an old instance of the consumer
    * by checking if there is a file called "pid" in the directory where
    * this class is run from.
    * @param args   A <code>String[]</code> with the arguments.
    */
   public static void main(String[] args)
   {
      /* References to the ORB and ocnsumer instance. */
      org.omg.CORBA.ORB orb = null;
      PersistentConsumer pc;

      /*
       * In order to make examples easier to run, for Orbacus-40 set a POA name.
       */
      if (com.prismt.openfusion.Version.getORB().
            toUpperCase().startsWith("ORBACUS-4"))
      {
         ObjectAdapter.setPOAName
            ("OpenFusion.EvaluationPersistencePushConsumer");
      }
      
      if ((com.prismt.openfusion.Version.getORB().
            toUpperCase().startsWith ("VISIBROKER-52")) || 
         (com.prismt.openfusion.Version.getORB().
            toUpperCase().startsWith ("VISIBROKER-60")))
      {
         defaultResolveName = "OFNotificationService";
      }

      /* The ORB name needs to be set for Orbix2000. */
      com.prismt.orb.ORBAdapter.setORBName("PersistentConsumer");

      /* Extract any command-line arguments we need for condiguration. */
      resolveName = extractArg (args, "-resolve", defaultResolveName);
      System.out.println("Notification service resolve name: " + resolveName);

      /*
       * Check whether a new consumer should be created or an old consumer
       * should be re-activated.  If there is a file called "pid", then there
       * is an old consumer and the file is read to determine the information
       * (the process ID and the IOR) needed.
       * 
       * The Process.PID system property (and any other properties as
       * appropriate) must be set before initialisng the ORB.  The
       * commented-out line of code is there to show that it is possible to set
       * the PID by instead making a direct call to PrismTech's "PID" class.
       */
      String oldConsumerRef = null;
      File f = new File("pid");
      if (f.exists ())
      {
         try
         {
            /* Open the file, and read the PID and consumer reference data. */
            BufferedReader br = new BufferedReader
               (new InputStreamReader(new FileInputStream(f)));
            String pidVal = br.readLine ();
            String refVal = br.readLine ();
            
            System.out.println("Setting Process.PID: " + pidVal);
            System.setProperty("Process.PID", pidVal);
            /* PID.setPID(new UUID(pidVal)); */

            oldConsumerRef = refVal;
         }
         catch(Exception e)
         {
            System.out.println ("An exception was thrown when reading the " +
                                "file with the old consumer's data.");
            e.printStackTrace();
            System.exit(1);
         }
      }

      /* Initialise the ORB. */
      try
      {
         orb = ObjectAdapter.init(args);
      }
      catch(Exception e)
      {
         System.out.println("An exception was thrown during ORB initialisation");
         e.printStackTrace();
         System.exit(1);
      }

      /* Create and initialise the persistent consumer. */
      pc = new PersistentConsumer (orb);
      pc.init(oldConsumerRef);

      /* Print out "Ready" to indicate we are ready to receive events. */
      System.out.println("Ready!");

      /* Ready the orb, and block so program does not exit. */
      ObjectAdapter.ready();
   }
}
