package com.prismt.cos.CosTime.examples.Event ;

import com.prismt.orb.* ;
import com.prismt.cos.CosTime.TimeUtil;

import org.omg.CosTime.* ;
import org.omg.CosTimerEvent.* ;
import org.omg.CosEventComm.*;
import org.omg.CosNaming.*;

import org.omg.CORBA.Any;

public class EventExample
{
   private static org.omg.CORBA.ORB orb = null;
   
   TimeService timeServer = null;      
   NamingContext nameServer = null;
   TimerEventService server = null;

   /**
    * Main method which creates a new EventExample object,
    * then initialises it and runs the example.
    */
   public static void main(String[] argv)
   {
      EventExample evex = new EventExample ();
      evex.init (argv);
      evex.go ();
   }

   /**
    * Initialises references to the ORB, the Time Service,
    * the Name Service and the TimerEvent Service.
    *
    * @param argv arguments to pass to the ORB initialisation
    */
   public void init (String [] argv)
   {
      System.out.println ("\n--Initialising example--");
      orb = com.prismt.orb.ORBAdapter.init (argv);      
      timeServer = getTimeServer ();
      nameServer = getNameServer ();
      server = getTimerEventServer ();
      System.out.println ("--Initialisation complete--");
   }   

   public void go ()
   {
      System.out.println ("\n--Running example--");

      /* Obtain a push consumer object */
      PushConsumer consumer = getPushConsumer ();
         
      /* Create an Any to use as an event */
      org.omg.CORBA.Any any = orb.create_any();
      any.insert_string ("I am a timed event!");
      
      /* register the event with the TimerEventService */
      TimerEventHandler handler = server.register (consumer, any);
      if (handler != null)
      {
         System.out.println
            ("\nEvent registered with the timer event service");
      }
      else
      {
         System.out.println ("Problem registering event with" 
                             + " the timer event service, Exiting");
         System.exit (1);
      }

      /* Send the event by three different methods */
      sendPeriodicEvent (handler, any);
      sendRelativeEvent (handler, any);
      sendAbsoluteEvent (handler, any);

      /* unregister the event with the TimerEventService */
      System.out.print ("\nUnregistering the handler... ");
      server.unregister (handler);
      System.out.println ("Unregistered.");

      System.out.println ("--Example complete--");
   }

   /**
    * Resolves a reference to a push consumer that has previously
    * been stored in the Name Service.
    *
    * @return A push consumer object.
    */
   public PushConsumer getPushConsumer ()
   {      
      System.out.print ("\nResolving push consumer from the Name Service...");
      NameComponent[] key = new NameComponent[1];
      key[0] = new NameComponent();
      key[0].id = "TimerEventConsumer";
      key[0].kind = "Push Consumer";
      
      org.omg.CORBA.Object o = null;
      
      try
      {   
         o = nameServer.resolve (key);
      }
      catch (Exception ex)
      {
         System.out.println (
            "Consumer not registered with the NamingService, Exiting " + ex);
         System.exit (1);
      }     
      
      PushConsumer consumer = PushConsumerHelper.narrow (o);
      System.out.println ("Resolved.");
      return consumer;
   }

   /**
    * Sends an event at an absolute time using the Timer Event Service.
    *
    * @param handler 
    */
   public void sendAbsoluteEvent (TimerEventHandler handler, Any any)
   {
      System.out.println ("\n-* Absolute Timed Event *-");
      
      /* change the event data */
      any.insert_string ("I am an absolute timed event!");
      handler.set_data (any);

      /* get the current time value for reference */ 
      UTO timenow = null;
      try
      {
         timenow = timeServer.universal_time ();
      }
      catch (TimeUnavailable ex)
      {
         ex.printStackTrace ();
      }

      /* set up the time for the event by adding
       * 5 seconds to the current time (5 000 millisecs)
       * [time service units are 1/10000 of a millisec]
       */

      TimeType ttype = TimeType.TTAbsolute;

      UTO newtime = timeServer.new_universal_time
      (
         timenow.time () + (5000L * 10000L), 
         timenow.inaccuracy (),
         timenow.tdf ()
      );

      /* see what the handler contains before the time is set */
      printDetails ("\n(Before Absolute Timer is set)", handler);

      /* set the time, then see what the handler contains now */
      handler.set_timer (ttype, newtime);
      printDetails ("\n(After Absolute Timer is set)", handler);

      System.out.println ("\nwait while event should be sent... ");

      try
      {
         Thread.currentThread().sleep (10000);
      }
      catch (Exception ex)
      {
         ex.printStackTrace ();
      }

      /* now see what handler contains after the event has been sent */
      printDetails ("\n(After Absolute Event has been sent)", handler);
   }

   public void sendRelativeEvent (TimerEventHandler handler, Any any)
   {
         
      System.out.println ("\n-* Relative Timed Event *-");

      /* change the event data */
      any.insert_string ("I am a relative timed event!");
      handler.set_data (any);

      /* get the current time value for reference */
      UTO timenow = null;
      try
      {
         timenow = timeServer.universal_time() ;
      }
      catch (TimeUnavailable ex)
      {
         ex.printStackTrace();
      }

      /* see what the handler contains before the timer is set */
      printDetails ("\n(Before Relative Timer is set)", handler);

      /* set timer for 5 seconds in the future */
      TimeType ttype = TimeType.TTRelative;
      long future = 5000L*10000L;
      UTO newtime = timeServer.new_universal_time
      (
         future, 
         timenow.inaccuracy(), 
         timenow.tdf()
      );

      handler.set_timer (ttype, newtime);

      /* see what the handler contains after the timer is set */
      printDetails ("\n(After Relative Timer is set)", handler);

      System.out.println ("\nwait while event should be sent... ");
      try
      {
         Thread.currentThread().sleep (7000);
      }
      catch (Exception ex)
      {
         ex.printStackTrace ();
      }

      /* see what the handler contains after the event is sent */
      printDetails ("\n(After Relative Event has been sent)", handler);
   }
   
   public void sendPeriodicEvent (TimerEventHandler handler, Any any)
   {
      System.out.println ("\n-* Periodic Timed Event *-");

      /* change the event data */
      any.insert_string ("I am a periodic timed event!");
      handler.set_data (any);
      
      /* get the current time value for reference */
      UTO timenow = null;
      try
      {
         timenow = timeServer.universal_time() ;
      }
      catch (TimeUnavailable ex)
      {
         ex.printStackTrace();
      }

      /* see what the handler contains before the timer is set */
      printDetails ("\n(Before Periodic Timer is set)", handler);

      /* create a periodic time of 2 seconds, and set the timer */
      TimeType ttype = TimeType.TTPeriodic;
      UTO newtime = timeServer.new_universal_time (
         2000*10000, timenow.inaccuracy(), timenow.tdf());
      handler.set_timer (ttype, newtime);

      /* see what the handler contains after the timer is set */
      printDetails ("\n(After Periodic Timer is set)", handler);

      System.out.println ("\nwait while events should be sent... ");
      try
      {
         Thread.currentThread().sleep (7000);
      }
      catch (Exception ex)
      {
         ex.printStackTrace ();
      }

      /* see what the handler contains after some events have been sent */
      printDetails ("\n(After Several Periodic Events have been sent)", 
                    handler);

      /* change the event data, DO NOT change the timer in any way */
      System.out.println ("\nChange the event data");
      any.insert_string ("I am a periodic timed event with new data!");
      handler.set_data (any);

      System.out.println ("\nwait while events should be sent... ");
      try
      {
         Thread.currentThread().sleep (8000);
      }
      catch (Exception ex)
      {
         ex.printStackTrace ();
      }

      /* see what the handler contains after more events have been sent */
      printDetails ("\n(After Several More Periodic Events have been sent)", 
                    handler);

      /* cancel the periodic timer */
      System.out.println("\nCancelling Periodic Event");
      boolean b = handler.cancel_timer ();
      
      /* see what the handler contains after the timer is cancelled */
      printDetails ("\n(After Cancelling Periodic Event)", handler);

      System.out.println ("\nwait a while - no more events should be sent... ");
      try
      {
         Thread.currentThread().sleep (4000);
      }
      catch (Exception ex)
      {
         ex.printStackTrace ();
      }

      /* see what the handler contains long after the timer is cancelled */
      printDetails ("\n(Long After Cancelling Event)", handler);
   }
   
   /**
    * Utility to print out description details of a Timer Event Handler.
    *
    * @param id identification string
    * @param handler the timer event handler
    */
   public void printDetails (String id, TimerEventHandler handler)
   {
      UTOHolder utoh = new UTOHolder ();
      boolean timeSet = handler.time_set (utoh);
      TimeType type = handler.time_type ();
      String typeString = "Unknown";
      EventStatus status = handler.status ();
      String statusString = "Unknown";
      
      System.out.println (id);
      System.out.println ("Time Set: " + timeSet);
      
      if (type.equals (TimeType.TTAbsolute))
      {
         typeString = "TTAbsolute";
         if (timeSet)
         {
            System.out.println ("    Time: " 
                                + TimeUtil.UTOtoDate(utoh.value));
         }
      }
      else if (type.equals (TimeType.TTRelative))
      {
         typeString = "TTRelative";
         if (timeSet)
         {
            System.out.println ("    Time: " + utoh.value.time()
                                + " x 100 nanoseconds");
         }         
      }
      else if (type.equals (TimeType.TTPeriodic))
      {
         typeString = "TTPeriodic";
         if (timeSet)
         {
            System.out.println ("    Time: " + utoh.value.time()
                                + " x 100 nanoseconds");
         }
      }
      System.out.println ("    Type: " + typeString);

      if (status.equals (EventStatus.ESTimeSet))
      {
         statusString = "ESTimeSet";
      }
      else if (status.equals (EventStatus.ESTimeCleared))
      {
         statusString = "ESTimeCleared";
      }
      else if (status.equals (EventStatus.ESTriggered))
      {
         statusString = "ESTriggered";
      }
      else if (status.equals (EventStatus.ESFailedTrigger))
      {
         statusString = "ESTimeFailedTrigger";
      }
      System.out.println ("  Status: " + statusString);
   }
   
   /*
    * Utility method to resolve the Time Service
    * 
    * @return reference to the time server
    */ 
   public TimeService getTimeServer ()
   {
      org.omg.CORBA.Object obj = null;  
      TimeService time = null;

      try
      {
         obj = orb.resolve_initial_references("TimeService");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println("Can't resolve Time Service, Exiting");
         System.exit(1);
      }

      time = TimeServiceHelper.narrow(obj);
      return time;
   }

   /*
    * Utility method to resolve the Name Service
    * 
    * @return reference to the name server
    */ 
   public NamingContext getNameServer ()
   {
      org.omg.CORBA.Object obj = null;  
      NamingContext name = null;

      try
      {
         obj = orb.resolve_initial_references("NameService");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println("Can't resolve Name Service, Exiting");
         System.exit(1);
      }

      name = NamingContextHelper.narrow(obj);
      return name;
   }
   
   /*
    * Utility method to resolve the Timer Event Service
    * 
    * @return reference to the timer event server
    */ 
   public TimerEventService getTimerEventServer ()
   {
      org.omg.CORBA.Object obj = null;  
      TimerEventService time = null;

      try
      {
         obj = orb.resolve_initial_references("TimerEventService");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println("Can't resolve Timer Event Service, Exiting");
         System.exit(1);
      }

      time = TimerEventServiceHelper.narrow(obj);
      return time;
   }   

}
