package com.prismt.cos.CosTime.examples.Time;

import org.omg.CosTime.*;
import com.prismt.cos.CosTime.TimeUtil;

public class TimeExample
{
   private TimeService server;

   /**
    * Main method which creates a new TimeExample object,
    * then initialises it and runs the example.
    */
   public static void main (String[] argv)
   {
      TimeExample example = new TimeExample ();
      example.init (argv);
      example.go ();
   }

   /**
    * Initialises references to the ORB and the Time Service.
    *
    * @param argv arguments to pass to the ORB initialisation
    */
   public void init (String[] argv)
   {
      System.out.println ("\n--Initialising example--");
      org.omg.CORBA.ORB orb = com.prismt.orb.ORBAdapter.init (argv);
      try
      {
         server = TimeServiceHelper.narrow
            (orb.resolve_initial_references ("TimeService"));
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex) {}
      if (server == null)
      {
         System.out.println ("Couldn't resolve time server; Exiting now.");
         System.exit (1);
      }
      System.out.println ("--Initialisation complete--");
   }

   /**
    * Runs the Time Service example.
    */  
   public void go ()
   {
      System.out.println ("\n--Running example--");
      printCurrentTime ();
      doTimedWorkDuringInterval (4);
      doTimedWorkUntilTime (4);
      System.out.println ("--Example complete--");
   }

   /**
    * Uses the time from the Time Service to obtain the current time
    * and print it out including details of the inaccuracy in it and 
    * the time zone.
    */
   public void printCurrentTime ()
   {
      System.out.println ("Printing Current Time");
      UTO timenow = null;
      try
      {
         timenow = server.universal_time ();
      }
      catch (TimeUnavailable ex)
      {
         System.err.println ("Time unavailable.");
      }

      if (timenow != null)
      {
         System.out.println
            ("Time now is " + timenow.time () + ", Error is " 
             + timenow.inaccuracy () + ", TZ is " + timenow.tdf () + " mins");
         System.out.println
            ("(" + TimeUtil.UTOtoDate (timenow).toString () + ")");
      }
      System.out.println ("Printed Current Time");
   }
   
   /**
    * Creates a time interval and does some work whilst the current
    * time lies within that time interval.
    *
    * @param seconds number of seconds long to make the time interval
    */
   public void doTimedWorkDuringInterval (int seconds)
   {
      System.out.println ("\nDoing some work for a " + seconds 
                          + " second interval.");
      TIOHolder dummy = new TIOHolder ();

      /* create a relative time with the required duration */
      UTO offset = server.new_universal_time 
         (seconds * 10000000, 0, (short) 0);

      /* convert relative to absolute time */
      UTO finish_time = offset.absolute_time ();

      try
      {
         /* make an interval representing the time 
          * during which we should work 
          */
         TIO working_interval =
            server.universal_time ().time_to_interval (finish_time);

         /* loop while the time is still enclosed in that interval */
         while (working_interval.spans (server.universal_time (), dummy)
                == OverlapType.OTContainer)
         {
            doWork ();
         }
      }
      catch (TimeUnavailable ex)
      {
         System.err.println ("Time unavailable.");
         return;
      }
      System.out.println ("Finished working for a " + seconds 
                          + " second interval.");
   }

   /**
    * Creates a time in the future and does some work whilst the current
    * time is less than that time.
    *
    * @param seconds number of seconds into the future to create the time
    */
   public void doTimedWorkUntilTime (int seconds)
   {
      System.out.println ("\nDoing some work until " + seconds 
                          + " seconds have passed.");

      /* create a relative time with the required duration */
      UTO offset = server.new_universal_time 
         (seconds * 10000000, 0, (short) 0);

      /* convert relative to absolute time */
      UTO finish_time = offset.absolute_time ();

      try
      {
         /* loop while finish_time > current time */

         while (finish_time.compare_time
                  (ComparisonType.MidC, server.universal_time ())
                == TimeComparison.TCGreaterThan)
         {
            doWork ();
         }
      }
      catch (TimeUnavailable ex)
      {
         System.err.println ("Time unavailable.");
         return;
      }

      System.out.println ("Finished the work as " + seconds 
                          + " seconds have passed.");
   }

   /**
    * Representing a method that does some work, this method:
    * <ul>
    *    <li> prints the current time (System time, not Time Service time)
    *    <li> sleeps for 500 milliseconds
    *    <li> prints the current time again
    * </ul>
    * this results in a time interval being printed out.
    * <p>
    * If the sleep is interrupted then a message saying so is printed.
    */
   private void doWork ()
   {
      System.out.print ((System.currentTimeMillis () / 100 % 600) / 10.0);
      try
      {
         Thread.sleep (500);
      }
      catch (InterruptedException ex)
      {
         System.out.print (" Interrupted !");
      }
      System.out.println
         (" - " + (System.currentTimeMillis () / 100 % 600) / 10.0);
   }

}
