package org.jacorb.imrutility.imr;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;

import org.omg.CORBA.ORB;
import org.omg.CORBA.INTERNAL;
import org.omg.PortableServer.*;


import org.jacorb.imrutility.imr.persistence.ActivatorData;
import org.jacorb.imrutility.imr.persistence.ActFilePersistence;
import org.jacorb.util.Debug;
import org.jacorb.util.Environment;

import org.apache.avalon.framework.logger.*;

/**
 * @author cj
 *
 * This class is used by the Implementation Repository to register the
 * start up details for server implementations.  Where configured to do
 * so the <code>Activator</code> is also used by the Implementation
 * Repository to start servers using the registered commands.
 *
 */
public class ActivatorImpl extends ActivatorPOA
{
    /** Local storage of the registered servers and start up commands*/
    private ActivatorData actData = null;

    /** The ID of this Activator */
    private long id;

    /** The time interval which controls when the Activator pings the IMR */
    private int heartbeat;

    /** The IOR location for the Activator */
    private static String activatorIOR;
    
    /** The file to be used for data persistence*/
    private static String dataFile;

    /** Reference to the ORB */
    private static ORB orb;
    
    /** Details of the commands registered with this Activator */
    private HashMap commands = new HashMap();
    
    /** Logs output */
    private Logger logger;

    private ActFilePersistence persistence = new ActFilePersistence();
    
    /** If true persisted data is not read in at start up*/
    protected static boolean newFile = false;

    /**
     * Constructor
     * @param <code> orb reference to the ORB for this Activator
     */
    public ActivatorImpl()
        throws Exception
    {
        logger = Debug.getNamedLogger("jacorb.activator");
           
        persistence = new ActFilePersistence();

        if (!newFile)
        {
            populateData();
        }
        else
        {
            actData = new ActivatorData();
        }

        Shutdown shutdownThread = new Shutdown();
        shutdownThread.setDaemon(true);
        shutdownThread.setName("Act Shutdown Thread");
        
        Runtime.getRuntime().addShutdownHook(shutdownThread);
        
        POA rootPOA = POAHelper.narrow
            (orb.resolve_initial_references("RootPOA"));

        rootPOA.the_POAManager().activate();

        org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[2];

        policies[0] =
            rootPOA.create_lifespan_policy (LifespanPolicyValue.PERSISTENT);

        policies[1] =
            rootPOA.create_id_assignment_policy
                (IdAssignmentPolicyValue.USER_ID);

        POA ActivatorPOA = rootPOA.create_POA("ActivatorPOA",
                                            rootPOA.the_POAManager(),
                                            policies );

        for (int i = 0; i < policies.length; i++)
        {
           policies[i].destroy();
        }

        byte[] id = "imr_activator".getBytes();

        ActivatorPOA.activate_object_with_id(id, this );

        PrintWriter pw = new PrintWriter
            (new FileOutputStream(new File(activatorIOR)));

        pw.println(
            orb.object_to_string(ActivatorPOA.servant_to_reference ( this )));

        pw.flush();
        pw.close();

        orb.run();
    }

    /**
     * Method to populate data from persistence if required
     */  
    private void populateData()
    {
        actData = persistence.read();
        commands = actData.getCommands();        
    }
    
    
    /**
     * Adds a new command.  The CommandInfo contains the <code>ServerName</code> 
     * used to identify the server and the command that is used to start the 
     * server e.g 
     * 
     *    jaco org.jacorb.naming.NameServer "$@". 
     * 
     * Where "$@" would be replaced with any parameters to be provided
     * at start up time.
     * 
     * @param cmd Reference to object holding the command details
     * @throws <code>AlreadyRegistered</code>
     */
    public void addCommand(CommandInfo cmd)
        throws AlreadyRegistered
    {

        POANameKey [] keys = new POANameKey().serverNameToKeys(cmd.name);
        Iterator iter;
        CommandData cdata;
        
        if (keys != null)
        {
            for (int i = 0; i < keys.length; i++)
            {  
                if (commands.containsKey(keys[i]))
                {                                  
                    throw new AlreadyRegistered(cmd.name);
                }
        
                cdata = new CommandData(cmd.cmd);
                                   
                commands.put
                (
                    keys[i], 
                    cdata
                );
             
                actData.addCommand
                (
                    keys[i], 
                    cdata
                );
                    
                persistence.write(actData);
            }
        }
    }

    /**
     * Removes any command registered under the given {@link ServerName}.
     * @param <code>sname</code> the keys for the required server 
     * @throws NotFound
     */
    public void removeCommand(ServerName sname)
        throws NotFound
    {
        /* Get all keys - if JacORB there will only be 1, for other ORBS there
         * may be multiple keys based on POA Names
         */        
        POANameKey [] keys = new POANameKey().serverNameToKeys(sname);
        Iterator iter;
                
        if (keys != null)
        {
            for (int i = 0; i < keys.length; i++)
            {
                if (!commands.containsKey(keys[i]))
                {                    
                    throw new NotFound
                        ("Command not found for " + keys[i].toString());
                }
                    
                commands.remove(keys[i]);
                actData.removeCommand(keys[i]);                                         
            }
                
            persistence.write(actData);
        }
        else
        {
            throw new NotFound("Command not found");
        
        }
    }

    /**
     * Returns details of commands held by this Activator
     * @return array of <code>CommandInfo</code> objects
     */
    public CommandInfo [] listCommands()
    {
        CommandInfo [] cmds = new CommandInfo[commands.size()];
        CommandInfo cinfo = null;
        CommandData rec;        
        Map.Entry entry;
        POANameKey pnKey;
        ServerName sname;
                
        Iterator iter = commands.entrySet().iterator();
        int i = 0;
        
        while (iter.hasNext())
        {
            entry = (Map.Entry) iter.next();

            pnKey = (POANameKey) entry.getKey();
            rec = (CommandData) entry.getValue();

            sname = pnKey.keyToServerName();        
            
            cinfo = 
                new CommandInfo(
                    sname,
                    rec.command);
                                  
            cmds[i] = cinfo;
            i++;
        }
        
        return cmds;
    }

    /**
     * Returns any command registered under the given {@link POANameKey}.
     * @param <code>pnkey</code> the unique key for the required server
     * @return {@link CommandInfo} the command details for the server
     */
    public CommandInfo [] listPOACommands(ServerName sname)
    {
        CommandData rec;       
        CommandInfo cinfo = null;
        CommandInfo [] cinfoArray;
        ArrayList alist = new ArrayList();
                 
        POANameKey [] keys = new POANameKey().serverNameToKeys(sname);
        
        if (keys != null)
        {
            for (int i = 0; i < keys.length; i++)
            {                        
                if (commands.containsKey(keys[i]))
                {
                    rec = (CommandData) commands.get(keys[i]);
                    
                    cinfo = 
                        new CommandInfo(
                            keys[i].keyToServerName(),
                            rec.command);                    
                        
                    alist.add(cinfo);
                }                
            }
        }               

        cinfoArray = new CommandInfo[alist.size()];
     
                
        return (CommandInfo []) alist.toArray(cinfoArray);    
    }
    

    /**
     * Starts the server using the <code>CommandInfo</code> object
     * held under the given {@link ServerName}.  The <code>shared</code>
     * flag is used to indicate whether the server is to be shared
     * among clients or not.  At present this flag is always true as
     * JacORB does not deal with PER_CLIENT requests.  This method
     * is currently only called if the <code>ActivationMode</code> 
     * is AUTO_START and multiples are NOT allowed.
     * @param name the name of the server
     * @param shared whether the request if for a shared server. For
     * the initial release this will always be true
     */
    public void startServer (ServerName sname, boolean shared)
        throws UnknownServer, StartupFailed
    {
        POANameKey [] keys = new POANameKey().serverNameToKeys(sname);
        CommandData crec;
        Iterator iter;
        POANameKey key;
                                  
        if (keys.length > 1)
        {
            throw new StartupFailed
                ("Cannot start multiple POAS");
        }

        crec = (CommandData) commands.get(keys[0]);
        
        if (crec != null)
        {            
            /* If there are no instances of the server starting */              
            if (crec.getServerCount() == 0)
            {
                if (logger.isDebugEnabled())
                {
                    logger.debug("Activator starting server " 
                        + keys[0].toString() 
                        + " and command " + crec.command);
                }
                 
                crec.add();
                   
                try
                {                    
                    Process proc = Runtime.getRuntime().exec(crec.command);
                }
                catch (Exception ex)
                {                    
                    throw new StartupFailed(
                        "Failed to start server " 
                        + keys[0].toString() + " :: " + ex.getMessage());
                }
            }
            else
            {
                throw new StartupFailed(
                    keys[0].toString()
                    + " :: Server already running");
            }
        }
        else
        {
            throw new UnknownServer(sname);
        }        
    }

    
    /**
     * Method used by the Repository to indicate that the start process
     * is complete.  This results in the server count being decremented
     * so any subsequent calls to startServer are allowed if there are
     * no servers in the process of being started.
     * @param sname the name of the server
     */
    public void serverStarted (ServerName sname)
        throws UnknownServer
    {
        POANameKey [] keys = new POANameKey().serverNameToKeys(sname);
        CommandData crec;
        
        crec = (CommandData) commands.get(keys[0]);

        if (crec != null)
        {            
            crec.remove();        
        }
        else
        {
            throw new UnknownServer(sname);
        }
    }

    /**
     * As the Activator is also a Server it may be pinged by the IMR to
     * ascertain whether it is still alive
     */
    public void ping(long id)
    {

    }

    /**
     * Shuts this server down
     * @param wait boolean indicating whether to wait for successful shutdown
     */
    public void shutdown(boolean wait)
    {
        persistence.backup(actData);
        orb.shutdown(wait);
    }

    /**
     * Access method to this objects ID
     */
    public long id()
    {
        return id;
    }

    /**
     * Sets this objects ID
     */
    public void id(long id)
    {
        this.id = id;
    }

    /**
     * Access method to the heartbeat variable.
     */
    public int heartbeat()
    {
        return heartbeat;
    }

    /**
     * Sets the heartbeat value
     */
    public void heartbeat(int heartbeat)
    {
        this.heartbeat = heartbeat;
    }

       
    /**
     * Main
     */
    public static void main(String args[])
    {
        Properties props = null;
        String dataFile = null;

        System.setProperty("jacorb.implname", "imr_activator");
        System.setProperty("jacorb.use_imr", "off");

        try
        {
            for (int i = 0; i < args.length ; i++)
            {
                if (args[i].equals("-h") || args[i].equals("--help"))
                {
                    usage();
                }
                
                if (args[i].equals("-n") 
                    || args[i].equals("--act_new"))
                {
                    if (dataFile != null)
                    {
                        System.out.println("New file not appropriate "
                           + "if data file specified");
                        System.exit(0);                                       
                    }
                    else
                    {
                        newFile = true;
                    }
                }
                else if (i < (args.length - 1))
                {
                    if (args[i].equals ("-i") || args[i].equals ("-act_ior"))
                    {
                        activatorIOR = args[++i];
                    }
                    else if (args[i].equals("-f") || args[i].equals("--act_data"))
                    {
                        if (newFile)
                        {
                            System.out.println("Data file name not appropriate "
                               + "if new file specified");
                            System.exit(0);                               
                        }
                        else
                        {
                            dataFile = args[++i];
                            Environment.setProperty
                                ("jacorb.imr.act_data_file", dataFile);
    
                        }
                    }
                    else if (args[i].equals("-b") 
                        || args[i].equals("--act_backup"))
                    {
                        Environment.setProperty
                            ("jacorb.imr.act_backup_file", args[++i]);
                    }                
                }
                else
                {
                    usage();
                }
            }
        }
        catch( Exception e )
        {
            e.printStackTrace();
            usage();
        }

        if (activatorIOR == null)
        {
            activatorIOR = 
                Environment.getProperty("jacorb.imr.activator_ior_file");

            if (activatorIOR == null)
            {
               System.out.println("IOR file name for Activator not known");
               usage ();
            }
        }
        
        orb = org.omg.CORBA.ORB.init(args, props);

        try
        {
            ActivatorImpl Activator = new ActivatorImpl();
        }
        catch (Exception ex)
        {
            System.out.println("Exception starting the Activator");
            ex.printStackTrace();
        }
    }


    /**
     * Help
     */
    private static void usage()
    {
        System.out.println(
            "--------------------------------------------------------------");
        System.out.println(
            "Usage:                                                        ");
        System.out.println();
        System.out.println(
            " [ -i | --act_ior ] <filename> The filename for the IOR for   ");
        System.out.println(
            "                               this Activator                 ");
        System.out.println(
            "--------------------------------------------------------------");
        System.out.println();
        System.out.println(
            " [ -f | --act_data ] <filename>The filename to be used for    ");
        System.out.println(
            "                               data persistence for this      ");
        System.out.println(
            "                               Activator                      ");
        System.out.println(
            "--------------------------------------------------------------");
        System.out.println
            ("[ -b | --act_backup ] <filename> The filename in which the   ");
        System.out.println
            ("                               Activator data is to be       ");
        System.out.println
            ("                               backed up - if required       ");
        System.out.println
            ("-------------------------------------------------------------");             
        System.out.println
            ("[ -n | --act_new ]            If set a new file will be      ");
        System.out.println
            ("                               created for the Activator data");
        System.out.println
            ("                               - if not set data in any      ");
        System.out.println
            ("                               existing file will be loaded  ");
        System.out.println
            ("-------------------------------------------------------------");                 

        System.out.println(
            " [ -h | --help ]               Displays this help             ");
        System.out.println(
            "--------------------------------------------------------------");

        System.exit (0);
    }

    /**
     * <code>Shutdown</code> is a thread that is run when receiving
     * a Ctrl-C or kill.
     */
    protected class Shutdown extends Thread
    {
        public synchronized void run()
        {
            if (logger.isDebugEnabled())
            {
                logger.info("ImR Activator: Shutting down");
            }

            shutdown(true);
        }
    }    
}
