package org.jacorb.imrutility.imr.persistence;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import org.jacorb.imrutility.imr.EndpointProfile;
import org.jacorb.imrutility.imr.POANameKey;
import org.jacorb.imrutility.imr.POARecord;
import org.jacorb.imrutility.imr.ServerRecord;
import org.jacorb.imrutility.imr.ImplRecord;

/**
 * @author cj
 *
 * The interface to be implemented by all objects responsible for storing 
 * <code>RepositoryImplementation</code> data.
 * @see org.jacorb.imrutility.imr.persistence.FilePersistence
 */
public class RepositoryData implements Serializable
{    
    /**
     * Map linking multiplicity and load balancing policy to an implementation.
     * Key can be impl name or POA names
     */
    protected Map impls;
    
    /**
     * Map linking server details to a unique server ID
     */
    protected Map servers;
    
    /**
     * Map linking POAs and Servers
     */
    protected Map poas;
    
    /**
     * Lock to control access to Maps
     */
    protected transient DataLock lock;
       
    /**
     * Map linking <code>Activator</code>s to implementations or POA Names
     */
    protected Map activators;    

    /**
     * Constructor
     */
    public RepositoryData()
    {        
        impls = Collections.synchronizedMap(new HashMap());
        servers = Collections.synchronizedMap(new HashMap());
        poas = Collections.synchronizedMap(new HashMap());
        activators = Collections.synchronizedMap(new HashMap());
        
        initTransient();
    }
 
    private void initTransient()
    {
        lock = new DataLock();
    }   
    
    /**
     * Add details of a server.
     * @param <code>sid</code> the unique ID of the Server
     * @param <code>srec</code> holds impl name and string representation of
     * <code>Server</code> object
     */
    public void addServer(long sid, ServerRecord srec)
    {
        lock.getLock(DataLock.SHARED);
        
        /* If an entry already exists for this server id the value is 
         * overwritten with the new ServerRecord
         */
        servers.put(new Long(sid), srec);    
        
        lock.releaseLock(DataLock.SHARED);
    }
    
    /**
     * Register details relating to a specific implementation e.g. NameService
     * @param <code>pnkey</code> the implementation name or POA Names
     * @param <code>irec</code> holds multiplicy and load balancing associated
     * with this impl
     */
    public void addImpl(POANameKey pnkey, ImplRecord irec)
    {
        lock.getLock(DataLock.SHARED);
        
        impls.put(pnkey, irec);    
       
        lock.releaseLock(DataLock.SHARED);       
    }


    /**
     * Register POAs for a specific server instance 
     * @param <code>sid</code> the unique ID of the server
     * @param <code>prect</code> a record containing a server id and endpoints
     */
    public void addPOA(POANameKey pnkey, POARecord prec)
    {
        lock.getLock(DataLock.SHARED);
        
        if (poas.get(pnkey) == null)
        {
            HashSet set = new HashSet();
            set.add(prec);
            poas.put(pnkey, set);
        }    
        else
        {
            ( (HashSet) poas.get(pnkey)).add(prec);
        }
        
        lock.releaseLock(DataLock.SHARED);
    }

    /**
     * Replace POA data - required where server details may have changed
     * so we just want to replace those details and not add a further
     * entry
     * @param <code>pnkey</code> the key of the POA
     * @param <code>prec</code> the new details
     */
    public void replacePOA(POANameKey pnkey, POARecord prec)
    {
        POARecord rec;
        Iterator iter;

        lock.getLock(DataLock.EXCLUSIVE);
              
        HashSet set = (HashSet) poas.get(pnkey);
        
        if (set != null)
        {
            iter = set.iterator();
            
            while(iter.hasNext())
            {
                rec = (POARecord) iter.next();
                
                if (rec.getID() == prec.getID())
                {
                    rec = prec;    
                }
            }
        }         
        
        lock.releaseLock(DataLock.EXCLUSIVE);
    }

    /**
     * Returns teh implementation details for a given POA. The
     * key may be either an impl name or the POA names but not
     * both. Implementation details can be preregistered and at
     * that time only an implementation name or POA names are
     * provided.
     * @param <code>pnkey</code> the implementation details key
     * @return the implementation record 
     */
    public ImplRecord getImpl(POANameKey pnkey)
    {
        return (ImplRecord) impls.get(pnkey);
    }
    
    /**
     * Method to retrieve the identities of any servers registered under
     * a given implementation name (impl name and/or POA names)
     * @param <code>impl</code> the implementation being queried
     * @return an array of server ids for the implementation
     */
    public Long [] getServerIDs(POANameKey pnkey)
    {
        return new Long[0];
    }

    /**
     * Returns the end point profiles for a given server
     * @param <code>serverID</code> the unique ID of the server
     * @return the EndpointProfiles for the requested server
     */
    public EndpointProfile[] getEndpoints(POANameKey pnkey, long serverID)
    {
        EndpointProfile [] endpoints = null;
        Iterator iter;
        POARecord prec;

        lock.getLock(DataLock.EXCLUSIVE);
        
        synchronized (poas)
        {
            HashSet set = (HashSet) poas.get(pnkey);
        
            if (set != null)
            {
                iter = set.iterator();
            
                while(iter.hasNext())
                {
                    prec = (POARecord) iter.next();
                
                    if (prec.getID() == serverID)
                    {
                        return prec.getEndpoints();
                    }
                }
            }         
        } 

        lock.releaseLock(DataLock.EXCLUSIVE);
                
        return endpoints;       
    }
    

    /**
     * Returns a <code>String</code> reference to the <code>Server</code>
     * object associated with the server ID provided
     * @param <code>serverID</code> the ID of the required server
     * @return <code>String</code> reference to the <code>Server</code> object
     */
    public ServerRecord getServer(long serverID)
    {
        return (ServerRecord) servers.get(new Long(serverID));        
    }

    /**
     * Returns the POA details for a server registered under a given key.
     * The key may be impl and POA names or just POA names.
     * @param <code>pnkey</code> the key for the POA
     * @param <code>serverID</code> the id of the required server
     * @return the <code>POARecord</code> for that server
     */    
    public POARecord getPOA(POANameKey pnkey, long serverID)
    {
        POARecord prec = null;
        
        lock.getLock(DataLock.EXCLUSIVE);

        synchronized (poas)
        {
            HashSet rpoas = (HashSet) poas.get(pnkey);
            Iterator iter = rpoas.iterator();
                    
            while (iter.hasNext())
            {
                prec = (POARecord) iter.next();
            
                if (prec.getID() == serverID)
                {
                    return  prec;        
                }
            }    
        }

        lock.releaseLock(DataLock.EXCLUSIVE);
                
        return prec;
    }

    /**
     * Removes all registrations under the given key
     */
    public void removePOA(POANameKey pnkey)
    {
        poas.remove(pnkey);        
    }    
    
    /**
     * Removes a number of POAs
     * @param <code>poaList</code> the list of POAs to be removed
     */
    public void removePOAs(ArrayList poaList)
    {
        for (int i = 0; i < poaList.size(); i++)
        {
            removePOA( (POANameKey) poaList.get(i));
        }
    }
    
    /**
     * Deregisters specified POAs for a given server
     * @param serverID the id of the server
     * @poaNames the names of the POAs to be removed
     */
    public void removePOA(long serverID, POANameKey pnkey)
    {
        HashSet set;
        Iterator iter;
        Set clone;
        POARecord prec;
        POANameKey key;
                    
        if (pnkey != null)
        {
            lock.getLock(DataLock.EXCLUSIVE);
            
            synchronized (poas)
            {
                set = (HashSet) poas.get(pnkey);
                iter = set.iterator();
            
                while (iter.hasNext())
                {
                    prec = (POARecord) iter.next();
                
                    if (prec.getID() == serverID)
                    {
                        iter.remove();
                    }
                }         
            }
                        
            lock.releaseLock(DataLock.EXCLUSIVE);
        }
        else
        {            
            /* need to get a clone of the set to avoid locking*/
            clone = (Set) poas.keySet();               
            iter = clone.iterator();
            
            while (iter.hasNext())
            {
                key = (POANameKey) iter.next();
                removePOA(serverID, key);                              
            }
        }          
    }
    

    /**
     * Deregisters all POAs for a given server id.
     * @param serverID the id of the server
     * @throws NotFound
     */
    public void removeServer(long serverID)
    {
        Long sID = new Long(serverID);
        ServerRecord srecord = (ServerRecord) servers.get(sID);
        
        if (srecord != null)
        {
            removePOA(serverID, null);
            servers.remove(sID);
        }
    }

    /**
     * Allowss removal of a number of servers.
     * @param <code>serverIDs</code> the list of ids to be removed
     */
    public void removeServers(ArrayList serverIDs)
    {
        for (int i = 0; i < serverIDs.size(); i++)
        {
            removeServer(( (Long) serverIDs.get(i)).longValue());
        }
    }

    /**
     * Method to remove an implementation
     * If any Servers registered for this implemenation are still active
     * then the implementation cannot be removed.
     * @param implName the name of the implementation
     * @throws NotFound ImplActive
     */
    public ImplRecord removeImpl (POANameKey pnkey)
    {
        return (ImplRecord) impls.remove(pnkey);
    }

    /**
     * Method to get the implementations table
     */
    public Map getImpls()
    {
        return impls;
    } 
       
    /**
     * Method to get the servers table
     */
    public Map getServers()
    {
        return servers;
    }

    /**
     * Method to get the POAs table
     */   
    public Map getPOAs()
    {
        return poas;
    }

    /**
     * Add an activator for a given implementation.  The <code>
     * POANameKey</code> will either be the impl name or the POA names
     * but not both.  Activators are pre-registered via the command line
     * and so impl name and POA names will never both be supplied.
     * NOTE: at this point only 1 activator can be registered however it 
     * is anticipated that there may be a requirement for more than one 
     * activator to be registered.  This will be added at a later date.
     * @param <code>id</code> the activators ID
     * @param <code>activator</code> the IOR of the activator.
     */ 
    public void addActivator(Long id, String activator)
    {
        activators.put(id, activator);
    }
    
    /**
     * Remove an activator for a given implementation.  The <code>
     * POANameKey</code> will either be the impl name or the POA names
     * but not both.  Activators are pre-registered via the command line
     * and so impl name and POA names will never both be supplied.    
     * @param <code>pnkey</code> the key to the implementation
     * @param <code>activator</code> the IOR of the activator
     */
    public void removeActivator(Long id, String activator)
    {
        activators.remove(id);
    }

    /**
     * Removes a number of activators.  Used when data is being loaded 
     * from persistence and an exception occurs when obtaining the
     * Activator object
     * @param <code>map</code> the Activator details to be removed
     */
    public void removeActivators(Map map)
    {
        Iterator iter = map.entrySet().iterator();
        Map.Entry entry;
        
        while (iter.hasNext())
        {
            entry = (Map.Entry) iter.next();
            
            removeActivator( (Long) entry.getKey(), (String) entry.getValue());    
        }
    }
    
    /**
     * Return all <code>Activator</code> details
     */
    public Map getActivators()
    {
        return activators;
    }
    
    /**
     * Implemented from the Serializable interface. For 
     * automatic initializing after deserialization.
     */
    private void readObject(java.io.ObjectInputStream in)
        throws java.io.IOException, java.io.NotActiveException, 
            ClassNotFoundException
    {
        in.defaultReadObject();
        initTransient();
    }    
    
}
    
   
