package org.jacorb.orb.giop;

/*
 *        JacORB - a free Java ORB
 *
 *   Copyright (C) 1997-2003  Gerald Brose.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// Imports
import java.util.LinkedList;
import java.util.List;
import org.apache.avalon.framework.logger.Logger;
import org.jacorb.orb.ORBConstants;
import org.jacorb.util.Debug;
import org.jacorb.util.Environment;
import org.omg.CORBA.INTERNAL;

/**
 * @author Nicolas Noffke
 * @version $Id: GIOPConnectionManager.java,v 1.5 2004/10/09 11:12:00 orbtest4 Exp $
 */

public class GIOPConnectionManager
{
    //private List client_giop_connections = null;
    private List server_giop_connections = null;
    private int max_server_giop_connections = 0;
    private Class statistics_provider_class = null;
    private SelectionStrategy selection_strategy = null;
    private int wait_for_idle_interval = 0;

    private Logger logger = Debug.getNamedLogger("jacorb.giop.conn");

    /**
     * <code>serviceContextKeyCache</code> is used to maintain a list of
     * ContextKey classes. These contain cached information that a ServerRequest
     * accesses upon instantiation.
     */
    private ContextKey []serviceContextKeyCache;
    /**
     * <code>svcId</code> is an index into serviceContextKeyCache.
     *
     */
    private int svcId;


    public GIOPConnectionManager()
    {
        server_giop_connections = new LinkedList();

        max_server_giop_connections =
            Environment.getIntPropertyWithDefault(
                "jacorb.connection.max_server_connections",
                Integer.MAX_VALUE );

        selection_strategy = (SelectionStrategy)
            Environment.getObjectProperty(
                "jacorb.connection.selection_strategy_class" );

        wait_for_idle_interval =
            Environment.getIntPropertyWithDefault(
                "jacorb.connection.wait_for_idle_interval", 500 );

        if( Environment.hasProperty(
            "jacorb.connection.statistics_provider_class" ))
        {
            String s = Environment.getProperty(
                "jacorb.connection.statistics_provider_class" );

            if( s != null && s.length() > 0 )
            {
                try
                {
                    statistics_provider_class =
                        Environment.classForName( s );
                }
                catch( Exception e )
                {
                    if (logger.isErrorEnabled())
                    {
                        logger.error( "Unable to create class from property >jacorb.connection.statistics_provider_class<: " + e.getMessage() );
                    }
                }
            }
        }

    }


    public ServerGIOPConnection createServerGIOPConnection(org.omg.ETF.Profile profile,
                                                           org.omg.ETF.Connection transport,
                                                           RequestListener request_listener,
                                                           ReplyListener reply_listener )
    {
        //if too many open connections, shut one down
        if( server_giop_connections.size() >= max_server_giop_connections )
        {
            if( selection_strategy != null )
            {
                while( server_giop_connections.size() >= max_server_giop_connections )
                {
                    ServerGIOPConnection to_close = null;

                    synchronized( server_giop_connections )
                    {
                        to_close =
                            selection_strategy.selectForClose( server_giop_connections );
                    }

                    if( to_close != null &&
                        to_close.tryClose() )
                    {
                        break;
                    }
                    else
                    {
                        try
                        {
                            Thread.sleep( wait_for_idle_interval );
                        }
                        catch( Exception e )
                        {
                            Debug.output( 1, e );
                        }
                    }
                }
            }
            else
            {
                if (logger.isErrorEnabled())
                {
                    logger.error( "No. of max server giop connections set, but no SelectionStrategy present" );
                }
            }
        }

        ServerGIOPConnection connection =
            new ServerGIOPConnection( profile,
                                      transport,
                                      request_listener,
                                      reply_listener,
                                      getStatisticsProvider(),
                                      this);

        synchronized( server_giop_connections )
        {
            server_giop_connections.add( connection );
        }

        return connection;
    }

    public void unregisterServerGIOPConnection(
        ServerGIOPConnection connection )
    {
        synchronized( server_giop_connections )
        {
            server_giop_connections.remove( connection );
        }
    }

    public GIOPConnection createClientGIOPConnection(
        org.omg.ETF.Profile profile,
        org.omg.ETF.Connection transport,
        RequestListener request_listener,
        ReplyListener reply_listener )
    {
        return new ClientGIOPConnection( profile,
                                         transport,
                                         request_listener,
                                         reply_listener,
                                         null );
    }

    private StatisticsProvider getStatisticsProvider()
    {
        StatisticsProvider result = null;
        if( statistics_provider_class != null )
        {
            try
            {
                result = (StatisticsProvider)
                    statistics_provider_class.newInstance();
            }
            catch( Exception e )
            {
                if (logger.isErrorEnabled())
                {
                    logger.error
                    (
                        "Unable to create instance from Class " +
                        statistics_provider_class
                    );
                }
            }
        }
        return result;
    }


    /**
     * <code>addContextKey</code> caches the key, oid and scopes from ServerRequest
     * in order to speed communication between JacORB implementations.
     *
     * @param object_key a <code>byte[]</code> value
     * @param oid a <code>byte[]</code> value
     * @param scopes a <code>List</code> value
     * @return a <code>byte[]</code> value
     */
    byte[] addContextKey(byte []object_key, byte []oid, List scopes)
    {
        byte []result = new byte[7];

        System.arraycopy(ORBConstants.JAC_TAG, 0, result, 0, 3);

        svcId++;

        if (serviceContextKeyCache == null)
        {
            // Should this be configurable?
            serviceContextKeyCache = new ContextKey[1000];
        }
        // Ooops, full so resize
        else if (svcId >= serviceContextKeyCache.length)
        {
            resizeCtxCache();
        }
        else if (serviceContextKeyCache[svcId] != null)
        {
            // Need to search forward to find another and possibly resize.
            while (serviceContextKeyCache[svcId] != null &&
                   svcId < serviceContextKeyCache.length)
            {
                svcId++;
            }
            // This means we were close to the end before, didn't find
            // any spare gaps and then hit the end. Can only resize now.
            if (svcId >= serviceContextKeyCache.length)
            {
                resizeCtxCache();
            }
        }

        result[3] = (byte)((svcId >> 24) & 0xFF);
        result[4] = (byte)((svcId >> 16) & 0xFF);
        result[5] = (byte)((svcId >>  8) & 0xFF);
        result[6] = (byte) (svcId        & 0xFF);

        if (logger.isDebugEnabled())
        {
            logger.debug("Adding context at index " + svcId + " with scopes content " + scopes + " for key " + new String(object_key));
        }

        serviceContextKeyCache[svcId] = new ContextKey (object_key, oid, scopes);

        return result;
    }


    /**
     * <code>resizeCtxCache</code> is used by addContextKey if the
     * serviceContextKeyCache reaches maximum size.
     */
    private void resizeCtxCache()
    {
        // Need to resize the array
        ContextKey []temp = new ContextKey[serviceContextKeyCache.length * 2];
        System.arraycopy
            (serviceContextKeyCache, 0, temp, 0, serviceContextKeyCache.length);
        serviceContextKeyCache = temp;
    }


    /**
     * <code>removeContextKey</code> is called when the GIOPConnection is closed.
     * It will remove the key at index.
     *
     * @param index an <code>int</code> value
     */
    void removeContextKey(int index)
    {
        if (serviceContextKeyCache[index] != null)
        {
            if (logger.isDebugEnabled())
            {
                logger.debug("Removing context at index " + index + " with scopes content " + (serviceContextKeyCache[index]).getScopes());
            }
            serviceContextKeyCache[index] = null;
        }
        else
        {
            logger.warn("ContextKey element was already null for index " + index);
        }
        // Reset svcId so we reuse empty slots.
        svcId = index - 1;
    }



    /**
     * <code>getContextKey</code> will return a cached ContextKey value
     * for that mini-key if one exists. It will check the byte key is in
     * the mini-key format.
     *
     * @param key a <code>byte[]</code> value
     * @return a <code>ContextKey</code> value
     */
    ContextKey getContextKey(byte []key)
    {
        ContextKey result = null;

        if (serviceContextKeyCache != null &&
            key != null                    &&
            key.length == 7                &&
            key[0]=='J'                    &&
            key[1]=='A'                    &&
            key[2]=='C'                     )
        {
            // Need to extra the id from the last 4 bytes of the key to get
            int index = (((key[3] & 0xff) << 24) +
                         ((key[4] & 0xff) << 16) +
                         ((key[5] & 0xff) <<  8) +
                         ((key[6] & 0xff) <<  0));

            try
            {
                result = (ContextKey)serviceContextKeyCache[index];
            }
            catch (ArrayIndexOutOfBoundsException e)
            {
                throw new INTERNAL
                (
                    "Cache out of bounds - unable to retrieve cached key entry for " +
                    index
                );
            }
            if (result == null)
            {
                // Client is sending me a mini-key but I can't find a
                // corresponding entry in the cache.
                throw new INTERNAL
                    ("Unable to retrieve cached key entry for " + index);
            }
        }
        if (logger.isDebugEnabled())
        {
            logger.debug
            (
                "getContextKey returning scopes" +
                (result == null ? "<null>" : (result.scopes).toString()) +
                " and key " +
                (result == null ? "<null>" : new String(result.key))
            );
        }
        return result;
    }


    /**
     * <code>getSvcID</code> returns the current value of svcId; it is used
     * by ServerGIOPConnection to store which slots that it is currently using.
     *
     * @return an <code>int</code> value
     */
    public int getSvcID()
    {
        return svcId;
    }


    /**
     * <code>ContextKey</code> is a wrapper class to cache the information
     * in serviceContextKeyCache.
     */
    public static final class ContextKey
    {
        private List scopes;
        private byte []key;
        private byte []oid;

        /**
         * Creates a new <code>ContextKey</code> instance. Called in
         * addContextKey.
         *
         * @param _key a <code>byte[]</code> value
         * @param _oid a <code>byte[]</code> value
         * @param _scopes a <code>List</code> value
         */
        public ContextKey (byte []_key, byte []_oid, List _scopes)
        {
            key    = _key;
            scopes = _scopes;
            oid    = _oid;
        }

        /**
         * <code>getScopes</code> returns the scopes value.
         *
         * @return a <code>List</code> value
         */
        public List getScopes()
        {
            return scopes;
        }

        /**
         * <code>getOID</code> returns the oid value.
         *
         * @return a <code>byte[]</code> value
         */
        public byte[] getOID()
        {
            return oid;
        }

        /**
         * <code>getKey</code> returns the key value.
         *
         * @return a <code>byte[]</code> value
         */
        public byte[] getKey()
        {
            return key;
        }
    }
}// GIOPConnectionManager
