package org.jacorb.orb.giop;

/*
 *        JacORB - a free Java ORB
 *
 *   Copyright (C) 1999-2003 Gerald Brose
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
import java.util.ListIterator; 

import org.omg.IOP.*;

import org.jacorb.orb.CDROutputStream; 
import org.jacorb.orb.ImRReadyProfile;
import org.jacorb.orb.TaggedComponentList;

/**
* A base class for IOP profiles.
* <p>
* @todo insert a better description. 
* @author Simon McQueen
* @version $Id: ProfileBase.java,v 1.2 2004/02/03 16:11:26 sm Exp $
*/
public abstract class ProfileBase extends org.omg.ETF._ProfileLocalBase
                         implements ImRReadyProfile, Cloneable
{
    protected TaggedComponentList  components = null;
    protected org.omg.GIOP.Version version = null;
    protected byte[]               objectKey = null;
    
    /**
     * This attribute shall contain the GIOP version number that this
     * profile supports. It is initialized each time an instance is 
     * created.
     */
    public org.omg.GIOP.Version version()
    {
        return version;
    }

    /**
     * Set's the object key. Must be called before marshal.
     * @todo I expect this to be removed from the API.
     */
    public void set_object_key(byte[] key)
    {
        this.objectKey = key;
    }
    
    /**
    * Gets the object key. Generally used after Profile has been created
    * by an ETF::Factories implementation.
    */
    public byte[] get_object_key()
    {
        return objectKey;
    }
    
    /**
     * This function marshals the appropriate information for this
     * transport into the tagged profile.  ORBs will typically need 
     * to call the IOR interception points before calling marshal().
     * <p>
     * This particular implementation *should* work for any IOP
     * type protocol that encodes its profile_data as a CDR encapsulated
     * octet array as long as you have correctly implemented 
     * the {@link #encapsulation()}, {@link #getAddressProfile()}, and
     * {@link #getTaggedComponents()} methods. But, feel free to override 
     * it for the purpose of optimisation or whatever (see IIOPProfile 
     * for example). It should however, remain consistent with you implementation
     * of the above mentioned methods.
     */
    public void marshal (TaggedProfileHolder tagged_profile,
                         TaggedComponentSeqHolder components)
    {
        if (encapsulation() != 0)
        {
            // You're going to have to define your own marshal operation 
            // for littleEndian profiles.
            // The CDROutputStream only does big endian.
            // @todo Suitable error.
        }
        
        // Start a CDR encapsulation for the profile_data
        CDROutputStream profileDataStream = new CDROutputStream();
        profileDataStream.beginEncapsulatedArray();
        
        // Write the opaque AddressProfile bytes for this profile...
        byte[] addressProfile = getAddressProfile();
        profileDataStream.write_octet_array(addressProfile, 0, addressProfile.length);
        
        // ... then the object key
        profileDataStream.write_long(objectKey.length);
        profileDataStream.write_octet_array(objectKey,0,objectKey.length);
        
        switch( version.minor )
        {
            case 0 :
                // For GIOP 1.0 there were no tagged components
                break;
            default :
                // Assume minor != 0 means 1.1 onwards and encode the TaggedComponents
                if (components == null)
                {
                    components = new TaggedComponentSeqHolder (new TaggedComponent[0]);
                }
                // Write the length of the TaggedProfile sequence.
                profileDataStream.write_long(this.components.size() + components.value.length);
                
                // Write the TaggedProfiles (ours first, then the ORB's)
                for (int i = 0; i < this.components.asArray().length; i++)
                {
                    TaggedComponentHelper.write(profileDataStream, this.components.asArray()[i]);
                }
                for (int i = 0; i < components.value.length; i++)
                {
                    TaggedComponentHelper.write(profileDataStream, components.value[i]);
                }        
        }
        
        // Populate the TaggedProfile for return.
        tagged_profile.value = new TaggedProfile
        (
            this.tag(),
            profileDataStream.getBufferCopy()
        );
    }
    
    /**
    * Indicates the encapsulation that will be used by this profile
    * when encoding it's AddressProfile bytes, and which should subsequently
    * be used when marshalling all the rest of the TaggedProfile.profile_data.
    * Using the default CDROutputStream for the encapsulation this should always
    * be 0
    */
    public short encapsulation()
    {
        return ImRReadyProfile.BIG_ENDIAN_TAG_INTERNET_IOP;
    }
    
    public TaggedComponentList getComponents()
    {
        return components;
    }

    public Object getComponent (int tag, Class helper)
    {
        return components.getComponent (tag, helper);
    }
    
    public void addComponent (int tag, Object data, Class helper)
    {
        components.addComponent (tag, data, helper);
    }
    
    public void addComponent (int tag, byte[] data)
    {
        components.addComponent (tag, data);
    }
    
    public TaggedProfile asTaggedProfile()
    {
        TaggedProfileHolder result = new TaggedProfileHolder();
        this.marshal (result, null);
        return result.value;
    }
    
    public Object clone() throws CloneNotSupportedException
    {
    
        ProfileBase result = (ProfileBase)super.clone();  // bitwise copy

        result.version = new org.omg.GIOP.Version (this.version.major,
                                                   this.version.minor);

        if (this.objectKey != null)
        {
            result.objectKey = new byte [this.objectKey.length];
            System.arraycopy (this.objectKey, 0, result.objectKey, 0,
                              this.objectKey.length);
        }
        
        if (this.components != null)
        {
            result.components = (TaggedComponentList)this.components.clone();   
        }
        
        return result;
    }
    
    public int hash()
    {
        return hashCode();
    }
}