package org.jacorb.test.imrutility.imr;

import junit.framework.*;

import org.jacorb.imrutility.imr.*;

import java.io.*;

public class MultiplesNoAlivenessTest extends PersistentServerTestCase
{
    private String serverIOR;
    private final String ior = "server";
    private final String home = System.getProperty("user.home");
    private final String separator = System.getProperty("file.separator");
    private RepositoryAdmin rep;
    private Activator act;
    
    private final int FIFO = 2;
    private final int LIFO = 3;
    private final int ROUND_ROBIN = 1;
    
    private String name;
            
    public MultiplesNoAlivenessTest(String name, PersistentServerSetup setup)
    {
        super(name, setup);
        this.name = name;
    }

    public static Test suite()
    {
        TestSuite suite = new TestSuite ("Multiples No Aliveness Test");
                       
        PersistentServerSetup setup =
            new PersistentServerSetup
            ( 
                suite,
                "org.jacorb.test.imrutility.imr.Server"
            );
 
        suite.addTest(new MultiplesNoAlivenessTest("testMultiplesFIFO", setup));
        suite.addTest(new MultiplesNoAlivenessTest("testMultiplesLIFO", setup));
        suite.addTest(new MultiplesNoAlivenessTest("testMultiplesRandom", setup));                   
        suite.addTest(new MultiplesNoAlivenessTest("testMultiplesRoundRobin", setup));   
         
        // Return the setup, not the suite!
        return setup;
    }
 
    protected void tearDown()
    {
        System.err.println("********** tearDown for " + name);
        
        cleanUp();                 
    }
       
    public void testMultiplesFIFO()
    {
        testMultiples(LoadBalancingPolicy.from_int(2), 2);
    }   
 
    public void testMultiplesLIFO()
    {
        testMultiples(LoadBalancingPolicy.from_int(3), 2);
    }   

    public void testMultiplesRandom()
    {
        testMultiples(LoadBalancingPolicy.from_int(0), 2);
    }

    public void testMultiplesRoundRobin()
    {
        testMultiples(LoadBalancingPolicy.from_int(1), 3);        
    }

    private void testMultiples(LoadBalancingPolicy lbpolicy, int servers)
    {
        String [] args = null;
        ServerName sname = null;
        StringBuffer command = null;
        BufferedReader br = null;
        TestServer test = null;
        TestServer [] tests = new TestServer[servers];
        Process [] procs = new Process[servers];
        Process proc = null;
        String [] iors = new String[servers];
          
        ActivationMode mode = ActivationMode.from_int(0);    
                                                  
        try
        {        
            /* Delete all files so no old data hanging around*/
            TestUtility.removeIMRFiles();
            
            /* Wait for files to be removed*/
            Thread.sleep(1000);
        }
        catch (Exception ex)
        {
            /* Don't do anything as this is not a problem*/
        }

        //Start the IMR        
        try
        {                                 
            /* Set fixed port and also aliveness policy of ON_LOOKUP */
            args = new String[0];
            rep = TestUtility.startImR(args, null);
            
            assertTrue("Failed to get ImR", rep != null);
            
            sname= new ServerName();
            sname.impl("test_imr");
            
            rep.addImpl
                (new ImplInfo(sname, true, lbpolicy, mode), null);                            
        }
        catch(Exception ex)
        {
            fail("Exception getting ImR" + ex.getMessage());
        }

        /* get command to start server, start then stop server - gets IOR file*/        
        try
        {
            for (int i = 0; i < servers; i++)
            {
                command = setup.getStartCommand();
                command.append(" -i ");
                serverIOR = home + separator + ior + i + ".ior";
                
                iors[i] = serverIOR;
                
                command.append(serverIOR);
                        
                proc = TestUtility.startServer(command);
                        
                procs[i] = proc;
            }
                               
            assertTrue("Server not registered", rep.listServers(sname).length == servers);
        }
        catch (IOException ioe)
        {
            fail("Got IOException starting server " + ioe.getMessage());
        }
        catch (Exception ex)
        {
            fail("Got Exception starting server " + ex.getMessage());
        }   

        for (int i = 0; i < iors.length; i++)
        {
            try
            {        
                br = new BufferedReader(new FileReader(iors[i]));

                // get object reference from command-line argument file
                org.omg.CORBA.Object obj =
                   TestUtility.getImRORB().string_to_object(br.readLine());

                br.close();

                //narrow to correct interface
                test = TestServerHelper.narrow(obj);

                tests[i] = test;
            
                test.ping();
            }
            catch (Exception ex)
            {
                fail
                    ("Unexpected exception pinging server " 
                        + i + " " + ex.getMessage());      
            }
        }        
                
        switch(lbpolicy.value())
        {
            case FIFO :
                procs[0].destroy();
                    
            case LIFO :
                procs[procs.length - 1].destroy();
                
            default :
                procs[1].destroy();
        }                    
                           
        int j = 0;            
                
        /* Try to ping the servers - the ping may or may not fail depending
         * on the load balancing policy
         */        
        while (j < 5)
        {
            for (int i = 0; i < tests.length; i++)
            {                       
                try
                {
                    tests[i].ping();                
                }
                catch (Exception ex)
                {
                    /* DO nothing - expected exception as there is no aliveness
                     * policy the ImR may return servers that are no longer 
                     * running.  In the case of FIFO and LIFO this will always 
                     * be the case.  With Random a running server may be returned if 
                     * there is a server running. RoundRobin will return the "next" 
                     * server which may be running or may not.  The servers will 
                     * not be removed from the ImR as the aliveness policy is 
                     * none.
                     */
                }       
            }
            
            j++;
        }            
       
        
        for (int i = 0; i < procs.length; i++)
        {            
            try
            {
                procs[i].destroy();
            }
            catch (Exception ex)
            {
            }
        }           
        
        for (int i = 0; i < iors.length; i++)
        {
            try
            {
                TestUtility.removeFile(iors[i]);
            }
            catch (Exception ex)
            {
                // Do nothing
            }
        }
        
        try
        {
            assertTrue("Incorrect number of servers", rep.listServers(sname).length == servers);
        }
        catch (NotFound nf)
        {
            fail("Server not found");
        }
    }
    
    private void cleanUp()
    {
        try
        {         
            if (rep != null)
            {       
                rep.shutdown(false);
            }
        
            TestUtility.removeIMRFiles();
            TestUtility.destroyImR();
            TestUtility.removeActivatorFiles();
            TestUtility.stopActivator();
            Thread.sleep(1000);
        }
        catch (Exception ex)
        {
            /* don't do anything */
        }                   
    }
}
