package org.jacorb.test.poa;

/*
 *        JacORB  - a free Java ORB
 *
 *   Copyright (C) 1997-2001  Gerald Brose.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.util.Properties;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.jacorb.orb.ParsedIOR;
import org.jacorb.orb.util.CorbaLoc;
import org.jacorb.test.orb.BasicServerImpl;
import org.omg.CORBA.Policy;
import org.omg.PortableServer.IdAssignmentPolicyValue;
import org.omg.PortableServer.LifespanPolicyValue;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAPackage.InvalidPolicy;

/**
 * <code>ImplNameTest</code> tests creating transient and persistent
 * POAs when using the implname property.
 *
 * @author <a href="mailto:rnc@prismtechnologies.com"></a>
 * @version 1.0
 */
public class ImplNameTest extends TestCase
{
    /**
     * <code>ImplNameTest</code> constructor - for JUnit.
     *
     * @param name a <code>String</code> value
     */
    public ImplNameTest (String name)
    {
        super (name);
    }


    /**
     * <code>suite</code> lists the tests for Junit to run.
     *
     * @return a <code>Test</code> value
     */
    public static Test suite ()
    {
        TestSuite suite = new TestSuite ("Implname Tests");

        suite.addTest (new ImplNameTest ("testNoImpl1"));
        suite.addTest (new ImplNameTest ("testNoImpl2"));
        suite.addTest (new ImplNameTest ("testNoImpl3"));
        suite.addTest (new ImplNameTest ("testNoImpl4"));
        suite.addTest (new ImplNameTest ("testNoImpl5"));

        return suite;
    }


    /**
     * <code>testNoImpl1</code> tests that we can create a transient POA
     * specifying an implname but the IOR still contains a random component
     */
    public void testNoImpl1 ()
    {
        Properties props = new Properties();
        props.setProperty("jacorb.implname", "TEST_RANDOM_COMPONENT");

        org.omg.CORBA.ORB orb = null;

        try
        {
            orb = org.omg.CORBA.ORB.init( new String []{}, props );

            POA rootPoa =
                (POAHelper.narrow( orb.resolve_initial_references( "RootPOA" )));

            // Create a child POA
            POA poa = rootPoa.create_POA
            (
                "TestServerPOA",
                rootPoa.the_POAManager(),
                new Policy[]
                {
                    rootPoa.create_id_assignment_policy( IdAssignmentPolicyValue.USER_ID)
                }
            );

            poa.the_POAManager().activate();

            // create the object reference
            poa.activate_object_with_id("Object".getBytes(), new BasicServerImpl());
            org.omg.CORBA.Object obj = poa.id_to_reference( "Object".getBytes() );

            ParsedIOR pior = new ParsedIOR( orb.object_to_string(obj), orb );

            assertTrue
            (
                CorbaLoc.parseKey( pior.get_object_key()).indexOf
                (props.getProperty("jacorb.implname")) == -1
            );
        }
        catch (Exception e )
        {
            fail("Incorrect exception " + e);
        }
        finally
        {
            orb.destroy();
        }
    }


    /**
     * <code>testNoImpl2</code> tests that we can create a persistent POA
     * specifying an implname and the IOR contains that component
     */
    public void testNoImpl2 ()
    {
        Properties props = new Properties();
        props.setProperty("jacorb.implname", "TEST_PERSISTENT_COMPONENT");

        org.omg.CORBA.ORB orb = null;

        try
        {
            // Environment needs to be reset because subsequent tests
            // reuse the same "instance" of the static Environment class.
            //org.jacorb.util.Environment.init();

            orb = org.omg.CORBA.ORB.init( new String []{}, props );

            POA rootPoa =
                (POAHelper.narrow( orb.resolve_initial_references( "RootPOA" )));

            // Create a child POA
            POA poa = rootPoa.create_POA
            (
                "TestServerPOA",
                rootPoa.the_POAManager(),
                new Policy[]
                {
                    rootPoa.create_lifespan_policy( LifespanPolicyValue.PERSISTENT),
                    rootPoa.create_id_assignment_policy( IdAssignmentPolicyValue.USER_ID)
                }
            );

            poa.the_POAManager().activate();

            // create the object reference
            poa.activate_object_with_id("Object".getBytes(), new BasicServerImpl());
            org.omg.CORBA.Object obj = poa.id_to_reference( "Object".getBytes() );

            ParsedIOR pior = new ParsedIOR( orb.object_to_string(obj), orb );

            assertTrue
            (
                CorbaLoc.parseKey( pior.get_object_key()).indexOf
                (props.getProperty("jacorb.implname")) != -1
            );
        }
        catch (Exception e )
        {
            e.printStackTrace();
            fail("Incorrect exception " + e);
        }
        finally
        {
            orb.destroy();
        }
    }


    /**
     * <code>testNoImpl3</code> tests that we can creating two transient objects
     * specifying the same impl and object id information name but the IOR will
     * still be different.
     */
    public void testNoImpl3 ()
    {
        Properties props = new Properties();
        props.setProperty("jacorb.implname", "TEST_RANDOM_COMPONENT_TWO");
        props.setProperty("jacorb.logfile.append", "on");

        org.omg.CORBA.ORB orb1 = null;
        org.omg.CORBA.ORB orb2 = null;

        try
        {
            orb1 = org.omg.CORBA.ORB.init( new String []{}, props );

            POA rootPoa =
                (POAHelper.narrow( orb1.resolve_initial_references( "RootPOA" )));

            // Create a child POA
            POA poa = rootPoa.create_POA
            (
                "TestServerPOA",
                rootPoa.the_POAManager(),
                new Policy[]
                {
                    rootPoa.create_lifespan_policy( LifespanPolicyValue.TRANSIENT),
                    rootPoa.create_id_assignment_policy( IdAssignmentPolicyValue.USER_ID)
                }
            );

            poa.the_POAManager().activate();

            // create the object reference
            poa.activate_object_with_id("Object".getBytes(), new BasicServerImpl());
            org.omg.CORBA.Object obj = poa.id_to_reference( "Object".getBytes() );

            ParsedIOR pior = new ParsedIOR( orb1.object_to_string(obj), orb1 );


            // Now create number two.

            // Environment needs to be reset because subsequent tests
            // reuse the same "instance" of the static Environment class.
            org.jacorb.util.Environment.init();

            orb2 = org.omg.CORBA.ORB.init( new String []{}, props );

            rootPoa = (POAHelper.narrow( orb2.resolve_initial_references( "RootPOA" )));

            // Create a child POA
            poa = rootPoa.create_POA
            (
                "TestServerPOA",
                rootPoa.the_POAManager(),
                new Policy[]
                {
                    rootPoa.create_lifespan_policy( LifespanPolicyValue.TRANSIENT),
                    rootPoa.create_id_assignment_policy( IdAssignmentPolicyValue.USER_ID)
                }
            );

            poa.the_POAManager().activate();

            // create the object reference
            poa.activate_object_with_id("Object".getBytes(), new BasicServerImpl());
            obj = poa.id_to_reference( "Object".getBytes() );

            ParsedIOR pior2 = new ParsedIOR( orb2.object_to_string(obj), orb1 );

            assertTrue
            (
                ! (CorbaLoc.parseKey( pior.get_object_key()).equals
                   (CorbaLoc.parseKey( pior2.get_object_key())))
            );
        }
        catch (Exception e )
        {
            fail("Incorrect exception " + e);
        }
        finally
        {
            orb1.destroy();
            orb2.destroy();
        }
    }



    /**
     * <code>testNoImpl4</code> tests that we cannot create a persistent POA without
     * specifying an implname.
     */
    public void testNoImpl4 ()
    {
        Properties props = new Properties();
        props.setProperty("jacorb.implname", "");

        org.omg.CORBA.ORB orb = null;

        try
        {
            orb = org.omg.CORBA.ORB.init( new String []{}, props );

            POA rootPoa =
                (POAHelper.narrow( orb.resolve_initial_references( "RootPOA" )));

            // Create a child POA
            POA poa = rootPoa.create_POA
            (
                "TestServerPOA",
                rootPoa.the_POAManager(),
                new Policy[]
                {
                    rootPoa.create_lifespan_policy( LifespanPolicyValue.PERSISTENT),
                    rootPoa.create_id_assignment_policy( IdAssignmentPolicyValue.USER_ID)
                }
            );
        }
        catch (InvalidPolicy e )
        {
            // Correct exception
            return;
        }
        catch (Exception e)
        {
            fail("Incorrect exception " + e);
        }
        finally
        {
            orb.destroy();
        }

        fail( "No exception");
    }


    /**
     * <code>testNoImpl5</code> tests that we can create a transient POA without
     * specifying an implname.
     */
    public void testNoImpl5 ()
    {
        Properties props = new Properties();
        props.setProperty("jacorb.implname", "");

        org.omg.CORBA.ORB orb = null;

        try
        {
            // Environment needs to be reset because subsequent tests
            // reuse the same "instance" of the static Environment class.
            org.jacorb.util.Environment.init();

            orb = org.omg.CORBA.ORB.init( new String []{}, props );

            POA rootPoa =
                (POAHelper.narrow( orb.resolve_initial_references( "RootPOA" )));

            // Create a child POA
            POA poa = rootPoa.create_POA
            (
                "TestServerPOA",
                rootPoa.the_POAManager(),
                new Policy[]
                {
                    rootPoa.create_id_assignment_policy( IdAssignmentPolicyValue.USER_ID)
                }
            );
        }
        catch (Exception e)
        {
            fail("Incorrect exception " + e);
        }
        finally
        {
            orb.destroy();
        }
    }
}
