/* Common tables */

CREATE TABLE OF_Servers
(
   SIDIndex NUMBER NOT NULL, /* server id index */
   SID CHAR(36) NOT NULL, /* server id uuid string */
   PRIMARY KEY (SID)
)
;

CREATE TABLE OF_Names
(
   Name LONG NOT NULL, /* name string */
   NameHash NUMBER NOT NULL, /* name hash value */
   NameIndex NUMBER NOT NULL, /* name index */
   PRIMARY KEY (NameIndex)
)
;

CREATE TABLE OF_Objects
(
   OID CHAR(36) NOT NULL, /* object id uuid string */
   PIDIndex NUMBER NOT NULL, /* process id index */
   TStamp DATE NOT NULL, /* update timestamp */
   Data LONG RAW NOT NULL, /* serialized object state */
   PRIMARY KEY (OID)
)
;

/* Procedure to find exisiting or create new process index */

CREATE PROCEDURE of_get_server_index
   (id IN VARCHAR2, idx OUT NUMBER)
IS
BEGIN
   SELECT SIDIndex INTO idx FROM OF_Servers WHERE SID = id;
EXCEPTION
      WHEN NO_DATA_FOUND
   THEN
      SELECT MAX (SIDIndex) + 1 INTO idx FROM OF_Servers;
      IF idx IS NULL
      THEN
         idx := 1;
      END IF;
      INSERT INTO OF_Servers (SIDIndex, SID) VALUES (idx, id);
END of_get_server_index;
/

/* Procedure to get/add name to table and return its index */

CREATE PROCEDURE of_get_name_index
   (nm IN LONG, nh IN NUMBER, idx OUT NUMBER)
IS
   CURSOR get_name_index_cur IS
      SELECT Name, NameIndex from OF_Names WHERE NameHash = nh;
   gnic get_name_index_cur%ROWTYPE;
BEGIN
   FOR gnic IN get_name_index_cur
   LOOP
      IF nm = gnic.Name
      THEN
         idx := gnic.NameIndex;
         EXIT;
       END IF;
   END LOOP;
   IF idx IS NULL
   THEN
      SELECT MAX (NameIndex) + 1 INTO idx FROM OF_Names;
      IF idx IS NULL
      THEN
        idx := 1;
      END IF;
      INSERT INTO OF_Names (Name, NameHash, NameIndex) VALUES (nm, nh, idx);
   END IF;
END of_get_name_index;
/

/* Procedure to save object */

CREATE PROCEDURE of_save_object
   (id IN VARCHAR2, pid IN NUMBER, dat IN LONG RAW)
IS
BEGIN
   UPDATE OF_Objects SET TStamp = SysDate, Data = dat WHERE OID = id;
   IF SQL%NOTFOUND THEN
      INSERT INTO OF_Objects (OID, PIDIndex, TStamp, Data)
         VALUES (id, pid, SysDate, dat);
   END IF;
END of_save_object;
/

/* Procedure to get object state */

CREATE PROCEDURE of_get_object
   (id IN VARCHAR2, dat OUT LONG RAW)
IS
BEGIN
   SELECT Data INTO dat FROM OF_Objects WHERE OID = id;
EXCEPTION
   WHEN NO_DATA_FOUND
   THEN
      dat := NULL;
END of_get_object;
/

/* Procedure to delete object */

CREATE PROCEDURE of_delete_object
   (id IN VARCHAR2)
IS
BEGIN
   DELETE FROM OF_Objects WHERE OID = id;
END of_delete_object;
/

/* Procedure to delete objects */

CREATE PROCEDURE of_delete_objects
   (pid IN NUMBER)
IS
BEGIN
   DELETE FROM OF_Objects WHERE PIDIndex = pid;
END of_delete_objects;
/

/* Procedure to check whether an object exists */

CREATE PROCEDURE of_object_exists
   (id IN VARCHAR2, ret OUT NUMBER)
IS
BEGIN
   SELECT PIDIndex INTO ret FROM OF_Objects WHERE OID = id;
EXCEPTION
   WHEN NO_DATA_FOUND
   THEN
      ret := 0;
END of_object_exists;
/
