package org.jacorb.imrutility.imr.handler;

import org.jacorb.imrutility.imr.Repository;
import org.jacorb.imrutility.imr.RepositoryHelper;

import org.omg.CORBA.ORB;
import org.omg.CORBA.TRANSIENT;
import org.omg.CORBA.ORBPackage.InvalidName;

import org.jacorb.util.Debug;
import org.jacorb.orb.giop.RequestInputStream;
import org.jacorb.orb.dsi.ServerRequest;
import org.jacorb.orb.ParsedIOR;
import org.jacorb.poa.util.POAUtil;

import java.util.List;

import org.apache.avalon.framework.logger.*;

/**
 * @author cj
 * 
 * This class is used to handle requests directed to the ImR that 
 * are not actually for the ImR.  A <code>locate</code> call is 
 * made to the ImR supplying details of the object that the 
 * request is really for or an alternative.
 *
 */
public class IMRRequestHandler
{
    /* Reference to the OpenFusion Repository */
    private Repository rep;

    /* The ORB */
    private ORB orb;

    private Logger logger;

    /* Constructor */
    public IMRRequestHandler( org.omg.CORBA.ORB orb )
    {        
        logger = Debug.getNamedLogger("jacorb.imr");
        this.orb = orb;        
    }

    /*
     * Receives requests that have been sent via the ORB but are not for
     * the ImR.  A locate request is sent to the IMR to obtain the 
     * required endpoints (or an alternative if the original target object 
     * is no longer available.
     * @param info the <code>ServerRequestInfo</code> for this request
     */
    public void handle_request (ServerRequest request)
    {
        String [] poaName = null;
        List pnames = null;
        String pname = null;

        if (logger.isDebugEnabled())
        {
            logger.debug
                ("IMRRequestHandler: handle_request for " + request.operation());
        }

        /*
         * Get the object key from the ServerRequest. This will have had any 
         * dereferencing of the object aliases from the object key map done.
         */
        byte [] objKey = request.objectKey();

        /* Get the ImR reference if required */
        if (rep == null)
        {            
            try
            {
                rep = RepositoryHelper.narrow
                         (orb.resolve_initial_references
                             ("ImplementationRepository"));
            }
            catch (InvalidName in)
            {
                throw new TRANSIENT
                (
                    "Unable to locate ImR",
                    0,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO
                );
            }
            
            if (rep == null)
            {
                rep = ( (org.jacorb.orb.ORB) orb).getImR();
            }                        
        }

        /*
         * Decode the object key to extract the POA name sequence
         * @TODO something more sophisticated based on the fuller TargetAddress 
         * types, if available or at least fall back on the TAO algorithm if we 
         * fail to locate based on this name.
         */
        pname = POAUtil.extractPOAName(objKey);
        pnames = POAUtil.extractScopedPOANames(pname);

        poaName = new String[pnames.size()];

        pnames.toArray(poaName);

        String implName = POAUtil.extractImplName(objKey);

        org.jacorb.imrutility.imr.EndpointProfile[] forwardEndpoints = null;

        try
        {
            forwardEndpoints = rep.locate(implName, poaName);
        }
        catch (Exception ex)
        {
            /*
             * Catch any exception as the following code will throw a
             * TRANSIENT if the target is null
             */
            if (logger.isDebugEnabled())
            {
                logger.debug
                    ("Caught exception locating " + implName + '/' + pname, ex);
            }
        }
        if (forwardEndpoints != null)
        {
             request.setLocationForward(( (org.jacorb.orb.ORB) orb).createIOR
                 ("IDL:omg.org/CORBA/Object:1.0", objKey, forwardEndpoints));

             if (logger.isDebugEnabled())
             {
                 logger.debug("IMRRequestHandler: Forwarding request.");
             }
        }
        else
        {
            /** Imr returned no endpoints. 'Throw' a transient. */
            request.setSystemException( new TRANSIENT
            (
                "IMR unable to locate target object",
                0,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO
            ));
        }

        /* Return the ForwardRequest or a Transient. */
        request.reply();
    }
}
