package org.jacorb.imrutility.imr.util;

import org.jacorb.imrutility.imr.ActivationMode;
import org.jacorb.imrutility.imr.Activator;
import org.jacorb.imrutility.imr.ActivatorHelper;
import org.jacorb.imrutility.imr.AlreadyRegistered;
import org.jacorb.imrutility.imr.ImplActive;
import org.jacorb.imrutility.imr.ImplInfo;
import org.jacorb.imrutility.imr.LoadBalancingPolicy;
import org.jacorb.imrutility.imr.NotFound;
import org.jacorb.imrutility.imr.POAInfo;
import org.jacorb.imrutility.imr.RepositoryAdmin;
import org.jacorb.imrutility.imr.RepositoryAdminHelper;
import org.jacorb.imrutility.imr.ServerInfo;
import org.jacorb.imrutility.imr.ServerState;
import org.jacorb.imrutility.imr.ServerName;
import org.jacorb.poa.POAConstants;

import java.util.StringTokenizer;
import java.util.Vector;

/**
 * @author cj
 *
 * This class is used to interrogate the OpenFusion Implementation
 * Repository from the command line.  It handles listing of Activators,
 * servers and poas.  Adding, removing and deactivaton of servers is 
 * also possible.
 */
public class IMRManager
{
    /** 
     * The ORB
     */
    private static org.omg.CORBA.ORB orb;

    /**
     * Array of string values for server status - used to display 
     */
    private static final String [] states =
        new String [] {"ACTIVE", "INACTIVE", "UNKNOWN"};

    /**
     * Array of string values for load balancing policy - used to display 
     */
    private static final String [] policies =
        new String [] {"Random", "RoundRobin", "FIFO", "LIFO"};

    /**
     * Array of string values for load balancing policy - used to display 
     */
    private static final String [] modes =
        new String [] {"NORMAL", "MANUAL", "PER_CLIENT", "AUTO_START"};
        
    /**
     * Reference to the Implementation Repository
     */
    private static RepositoryAdmin  rep = null;

    /**
     * Used in stringing/usntringing POA names
     */
    private static final char [] oks = {POAConstants.OBJECT_KEY_SEPARATOR};
       
    /**
     * Converts char [] to string for output purposes
     */ 
    private static final String OBJECT_KEY_SEPARATOR = new String(oks);
    
    /**
     * Main
     */
    public static void main(String args[])
    {
        System.setProperty("jacorb.implname", "ImR Manager");
        
        orb = org.omg.CORBA.ORB.init(args, null);

        if (args.length > 0)
        {
            /* Need to cater for a specific IMR which will be set using
             * the -ORBInitRef argument  So we need to loop through the
             * args in case it is there.
             */
 
            if (args[0].equals("-h") || args[0].equals("--help"))
            {
                usage(); // exits
            }
            else
            {
                getAdmin();

                /* Need to loop through in case -ORBInitRef set */
                for (int i = 0; i < args.length; i++)
                {
                    if (args[i].equals("-l") || args[i].equals("--list"))
                    {
                        list(args);
                        break;
                    }
                    else if (args[i].equals("-a") || args[i].equals("--add"))
                    {
                        addImplementation(args);
                        break;
                    }
                    else if (args[i].equals("-r") || args[i].equals("--remove"))
                    {
                        removeImplementation(args);
                        break;
                    }
                    else if (args[i].equals("-d") || args[i].equals("--deactivate"))
                    {
                        deactivate(args);
                        break;
                    } 
                    else if (args[i].equals("-k") || args[i].equals("--shutdown"))
                    {
                        shutdown(args);
                        break;
                    }                   
                }
            }
        }
        else
        {
            usage();
        }
    }

    /**
     * Utiltiy method to list details of objects registered with the Repository
     * @param args input parameters needed to perform the request
     */
    private static void list(String [] args)
    {
        ServerName sname;
      
        try
        {
            for (int i = 0; i < args.length;  i++)
            {                           
                if (args[i].equals("-v") || args[i].equals("-activators"))
                {
                    try
                    {
                        listActivators(rep.listActivators());
                    }
                    catch (Exception ex)
                    {
                        System.out.println("List activators not yet implemented");
                    }
                    
                    System.exit(0);

                }
                
                if (args [i].equals("-s") || args [i].equals("--server"))
                {
                    if (i < args.length - 1)
                    {
                        try
                        {
                            listPOAs(rep.listPOAs(Long.parseLong (args[++i])));
                        }
                        catch (NumberFormatException nfe)
                        {
                            System.out.println 
                                ("Invalid server id - must be numeric");
                        }
                    }
                    else
                    {
                        System.out.println("You must provide the server ID");
                    }
                    
                    System.exit (0);
                }
                else if (args [i].equals ("-n") || args [i].equals ("--name"))
                { 
                    if (i < args.length - 1)
                    {                                       
                        sname = new ServerName();
                        sname.impl(args[++i]);
            
                        listServers (rep.listServers (sname));
                    }
                    else
                    {
                        System.out.println("You must provide the server name");
                    }
                    
                    System.exit (0);
                
                }
                else if (args[i].equals("-p") || args[i].equals("--poa"))
                {
                    System.out.println("This option is not yet implemented");
                    System.exit(0);
                }               
            }
            
 
            /* If we get to here we haven't listed anything specific so just
             * list all implementations
             */                           
            listImpls(rep.listAllImpls());
            System.exit(0);

        }
        catch (Exception ex)
        {
            System.out.println 
                ("Caught an exception trying to perform request");
            ex.printStackTrace();
        }
    }

    /**
     * Lists the implementations registered with the repository
     * @param impls details of the implementations registered
     */
    private static void listImpls(ImplInfo [] impls)
    {
        if (impls.length == 0)
        {
            System.out.println("\nNo implementations recorded");
        }
        else
        {
            ImplInfo iinfo;

            System.out.println ("\nImplementations\n");

            for (int i = 0; i < impls.length; i++)
            {
                iinfo = impls[i];
                System.out.println 
                    ( (i + 1)  + ". " + "Name                : " 
                        + getName(iinfo.name));
                   
                System.out.println 
                    ("   Multiples allowed   : " 
                        + getStringValue(iinfo.multiples));
                    
                System.out.println ("   LoadBalancingPolicy : "
                    + policies[iinfo.lbPolicy.value()]);
                    
                System.out.println ("   Activation Mode     : "
                    + modes[iinfo.activation.value()]);

                System.out.println ("\n\n");
            }
        }
    }

    /**
     * Lists the POAS registered with the repository
     * @param poas details of the POAs registered
     */
    private static void listPOAs(POAInfo [] poas)
    {
        if (poas.length == 0)
        {
            System.out.println("\nNo poas recorded");
        }
        else
        {
            POAInfo pinfo;

            System.out.println("\nPOAs\n");
            
            for (int i = 0; i < poas.length; i++)
            {
                pinfo = poas[i];
                
                System.out.println
                    ( (i + 1) + ". " + "Name                : " 
                        + pinfo.name);
                        
                System.out.println 
                    ("   LoadBalancingPolicy : "
                        + policies [pinfo.lbPolicy.value()]);
                        
                System.out.println ("\n\n");
            }
        }
    }

    /**
     * Lists the servers registered with the repository
     * @param servers details of the servers registered
     */
    private static void listServers(ServerInfo[] servers)
    {
        if (servers == null || servers.length == 0)
        {
            System.out.println("\nNo servers recorded");
        }
        else
        {
            ServerInfo sinfo;
            String status;
            String loc;
         
            System.out.println("\nServers\n");
          
            for (int i = 0; i < servers.length; i++)
            {
                sinfo = servers[i];

                status = states[sinfo.state.value()];
                
                loc = orb.object_to_string(sinfo.srv);
          
                System.out.println("   ID       : " + sinfo.id);
                System.out.println("   Status   : " + status);
                System.out.println("   Location : " + loc);
            
                System.out.println ("\n\n");
            }
        }
    }

    /**
     * Lists details of Activators registered with the ImR
     */
    private static void listActivators(ServerInfo [] activators)
    {
        // TODO
        
        /* The listing of Activators has not yet been implemented aa the
         * interface only allows a listing of All Activators and returns
         * a ServerInfo array.  This is not terribly useful as Activators
         * are registered for given implementations/POAs and therefore it
         * would be better to provide detials for a specific ServerName.
         * Also the Activator is not a Server it simply implements 
         * ServerOperations.  The Activator is not registered with the 
         * ImR so it doesn't get a ServerImpl created.
         * That would require changing the IDL and it is getting late in
         * the day to do that
         */        
        if (activators != null && activators.length == 0)
        {
            System.out.println ("\nNo activators recorded");
        }
        else
        {
            ServerInfo act;
            String loc;
         
            System.out.println("\nActivators\n");

            for (int i = 0; i < activators.length; i++)
            {
                act = activators[i];
                 
                loc = orb.object_to_string(act.srv);
          
                System.out.println(" ID       : " + act.id);
                System.out.println(" State    : " + states[act.state.value()]);
                System.out.println(" Location : " + loc);
            
                System.out.println("\n");
            }
        }      
    }
   
   
    /**
     * Utility method used to add an implementation to the repository
     * @param args the details to be registered, the name of the
     * implementation and whether multiples are allowed
     */
    private static void addImplementation(String [] args)
    {
        String name = null;
        boolean multiple = false;
        LoadBalancingPolicy policy = null;
        Activator act = null;
        String act_ior = null;
        ServerName sname = null;
        ActivationMode mode = null;
              
        try
        {
            for (int i = 0; i < args.length ; i++)
            {
                if (args[i].equals("-n") || args[i].equals("--name"))
                {
                    if (i < args.length - 1)
                    {
                        name = args[++i];
                    }
                }
                else if (args[i].equals("-p") || args[i].equals("--poa"))
                {
                    System.out.println("Addition of poa is not yet implemented");
                    System.exit(0);
                }
                else if (args[i].equals("-m")  || args[i].equals("--multiples"))
                {
                    
                    if (i < args.length - 1 && args[++i].equalsIgnoreCase("on"))
                    {
                        multiple = true;
                    }
                }
                else if (args[i].equals("-l") || args[i].equals("--lb_policy"))
                {
                    if (i < args.length - 1)
                    {
                        policy =
                            LoadBalancingPolicy.from_int 
                                (Integer.parseInt(args[++i]));
                    }
                }
                else if (args[i].equals("-v") || args[i].equals("--activator"))
                {
                    if (i < args.length - 1)
                    {
                        act_ior = args[++i];
                    }
                }    
                else if (args[i].equals("-e") || args[i].equals("--mode"))
                {
                    if (i < args.length - 1)
                    {
                        mode =
                            ActivationMode.from_int 
                                (Integer.parseInt(args[++i]));
                    }                        
                }        
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
            usage();
        }

        if (name == null)
        {
            System.out.println("\nYou must provide a server name\n");
            usage();
        }

        if (policy == null)
        {
            policy = LoadBalancingPolicy.from_int(0);
        }

        if (mode == null)
        {
            mode = ActivationMode.from_int(0);
        }
        
        try
        {            
            sname = getServerName(name);
            
            if (act_ior != null)
            {
                act = ActivatorHelper.narrow(orb.string_to_object(act_ior));  
            }
            
            rep.addImpl(new ImplInfo(sname, multiple, policy, mode), act);
            
            System.out.println(name + " added");
        }
        catch (AlreadyRegistered aa)
        {
            System.out.println
                ("The implemenation is already registered :: " + aa.name); 
        }
        catch (Exception ex)
        {
            System.out.println
                ("An exception occurred adding " + sname 
                    + " :: " + ex.getMessage());
        }
    }

    /**
     * Utility method to remove a specified implementation
     * @param name the name of the implementation to be removed
     */
    private static void removeImplementation(String[] args)
    {
        ServerName sname = null;
        String act_ior = null;
        Activator act = null;
        String name = null;

        for (int i = 0; i < args.length; i++)
        {
            if (args[i].equals("-n") || args[i].equals("--name"))
            {
                if (i < args.length - 1)
                {  
                    name = args[++i];
                    sname = getServerName(name);
                }
            }
            else if (args[i].equals("-p") || args[i].equals("--poa"))
            {
                System.out.println("Removal of POA is not yet implemented");
                System.exit(0);
            }
            else if (args[i].equals("-v") || args[i].equals("--activator"))
            {
                if (i < args.length - 1)
                {
                    act_ior = args[++i];
                }
            }
        }
         
        if (sname == null)
        {
            usage();            
        }           
                   
        try
        {
            if (act_ior != null)
            {
                act = ActivatorHelper.narrow(orb.string_to_object(act_ior));
            }
          
            rep.removeImpl(sname, act);
            
            System.out.println(name + " removed");
        }
        catch (ImplActive ia)
        {
            System.out.println
                ("\nYou cannot remove this implementation - it is still active");
        }
        catch (NotFound nf)
        {
            System.out.println("\nImplemenation not found");
        } 
    }
 
    /**
     * Currently the deactivation is at Server level to allow individual
     * servers to be removed rather than all servers.  The deactivation
     * of an Implementation code has been left in for now as it may be
     * required in the future
     */
    private static void deactivate(String [] args)
    {        
        for (int i = 0; i < args.length; i++)
        {
            if (args[i].equals("-s") || args[i].equals("--server"))
            {                
                if (i < args.length - 1)
                {
                    deactivateServer(args[++i]);                    
                }
                else
                {
                    System.out.println("You must provide the Server ID");
                }
            }
        }
    }
    
    /**
     * Utility method to remove a Server for a given ID
     * @param the ID of the Server to be removed
     */
    private static void deactivateServer(String id)
    {
        try
        {
            Long sID = new Long(id);    
            
            rep.deregisterAll(sID.longValue());
            
            System.out.println("Server id " + id + " deactivated");
        }
        catch (NumberFormatException nfe)
        {
            System.out.println("Invalid server ID - must be numeric");
        }
        catch (NotFound nf)
        {
            System.out.println("Server not found for " + id);
        }
    }
    
    /**
     * Utility method to deactivate a specified implementation - not handled
     * called at present.
     * @param name the name of the implementation to be set down
     */
    private static void deactivateImplementation(String name)
    {
        ServerName sname = getServerName(name);
                        
        try
        {                    
            ServerInfo[] servers = rep.listServers(sname);
            ServerInfo sinfo;

            for (int i = 0; i < servers.length; i++)
            {
                sinfo = servers[i];

                rep.deregisterAll(sinfo.id);
            }
        }
        catch (NotFound nf)
        {
            System.out.println("Implementation not found for " + name);
        }
    }

    /**
     * Shutdown method that results in a specified Server being shutdown
     */
    private static void shutdown(String [] args)
    {        
        ServerName sname = null;
        long serverID = 0;        
        ServerInfo [] sinfos = null;
        String name = null;
        
        for (int i = 0; i < args.length; i++)
        {            
            if (args[i].equals("-n") || args[i].equals("--name"))
            {
                if (i < args.length - 1)
                {
                    name = args[++i];
                    sname = getServerName(name);
                }
            }
            else if (args[i].equals("-p") || args[i].equals("--poa"))
            {
                System.out.println("POA shutdown is not yet implemented");
                System.exit(0);
            }
            else if (args[i].equals("-s") || args[i].equals("--server"))
            {
                if (i < args.length - 1)
                {
                    serverID = Long.parseLong(args[++i]);
                }
            }     
        }    

        try
        {
            if (sname != null)
            {
                sinfos = rep.listServers(sname);
            
                if (sinfos.length == 0)
                {
                    System.out.println("No servers to shutdown"); 
                    System.exit(0);
                }
                else if (sinfos.length > 1)
                {
                    if (serverID == 0)
                    {
                        System.out.println("Multiple servers registered");
                        System.out.println("Please provide server ID");
                        System.exit(0);
                    }
                    else
                    {
                        for (int i = 0; i < sinfos.length; i++)
                        {
                            if (serverID == sinfos[i].id)
                            {
                                sinfos[i].srv.shutdown(false);
                                break;
                            }
                        }
                    }
                }
                else
                {
                    sinfos[0].srv.shutdown(false);
                }
            
                System.out.println
                    ("Shutdown server " + name + " ID = " + serverID);
            }
            else
            {
                System.out.println("Please provide the server name");
            }           
        }
        catch (Exception ex)
        {
            System.out.println("Exception shutting down server " + sname);
            System.out.println(ex.getMessage());        
        }
    }
    
    /**
     * Method to get a reference to the RepositoryAdmin
     */
    private static void getAdmin()
    {
        try
        {
            rep = RepositoryAdminHelper.narrow
                (orb.resolve_initial_references("ImplementationRepository"));

            if (rep == null)
            {
                System.out.println("Unable to resolve the IMR");
                System.exit(0);
            }
        }
        catch (Exception ex)
        {
            System.out.println("Exception getting IMR");
            System.out.println(ex.getMessage());
            ex.printStackTrace();
            System.exit(0);
        }
    }

    /**
     * Utility method to return a value of "on" or "off" rather than "true"
     * or "false" when decoding a boolean.
     */
    private static String getStringValue(boolean active)
    {
        if (active)
        {
            return "on";
        }

        return "off";
    }

    /**
     * Gets the name to be displayed from a ServerName.  This may be
     * the impl name or POA Names but not both
     */  
    private static String getName(ServerName sname)
    {
        String name = null;
        String [] [] names = null;
        String [] pnames = null;
        
        if (sname.discriminator())
        {
            name = sname.impl();    
        }
        else
        {
            names = sname.poas();
                
            if (names != null)
            {        
                for (int i = 0; i < names.length; i++)
                {                
                    for (int j = 0; j < names[i].length; j++)
                    {
                        name += OBJECT_KEY_SEPARATOR;
                        name += names[i][j];
                    }
                }
            }
        }
        
        return name;
    }  

    /**
     * This converts an input String into a ServerName.  This is
     * required for instances when registrations use only the 
     * POA names.  That functionality is not yet implemented.
     */    
    private static ServerName getServerName(String in_name)
    {
        StringTokenizer st = new StringTokenizer(in_name, OBJECT_KEY_SEPARATOR);
        String name = null;
        String [] [] poas = null;
        String [] ipoas;
        Vector vpoas = new Vector();
        ServerName sname = new ServerName();
        
        if(in_name.startsWith(OBJECT_KEY_SEPARATOR))
        {
            while (st.hasMoreTokens())
            {
                vpoas.add(st.nextToken());
            }

            ipoas = new String[vpoas.size()];

            for (int i = 0; i < vpoas.size(); i++)
            {
                ipoas [i] = (String) vpoas.elementAt(i);
            }
            
            poas = new String [1] [ipoas.length];
            poas[0] = ipoas;
        }
        else
        {
            name = st.nextToken();
        }
        
        if (name != null)
        {
            sname.impl(name);
        }
        else
        {
            sname.poas(poas);
        }
        
        return sname;
    } 
    
    
    /**
     * Help
     */
    private static void usage()
    {
        System.out.println("---------------------------------------------------------");
        System.out.println("Use this tool to interrogate the Implementation Repository");
        System.out.println("or to pre-register/remove servers.                       ");
        System.out.println("Valid options are as follows:                            ");
        System.out.println();
        System.out.println(" [ -l | --list ]            Lists all registered         ");
        System.out.println("                            implementations              ");
        System.out.println();        
        System.out.println("The following options can be used with -l or --list      ");
        System.out.println();
        System.out.println(" [ -n | --name ] <name>     Lists all servers registered ");
        System.out.println("                            for a given implementation   ");
        System.out.println();
        System.out.println(" [ -s  | --server ] <server id> Lists all poas for a given ");
        System.out.println("                              server using the server id   ");
        System.out.println();
        System.out.println(" [ -p  | --poa ] <poa name> Not yet implemented          ");
        System.out.println();        
        System.out.println(" [ -v  | --activators ]     Lists all Activators known to");
        System.out.println("                            the ImR                      ");
        System.out.println("---------------------------------------------------------");
        System.out.println("To add or remove servers use the following commands.     ");
        System.out.println();
        System.out.println("   [ -a | --add ]        Adds implementation details     ");
        System.out.println();
        System.out.println("The following options can be used with -a or --add       ");
        System.out.println();
        System.out.println(" [ -n | --name ] <name>     The server name (mandatory)  ");
        System.out.println();
        System.out.println(" [ -p  | --poa ] <poa name> Not yet implemented          ");
        System.out.println();
        System.out.println(" [ -m | --multiples ] <ON/OFF>                           ");
        System.out.println("                         whether multiple servers are allowed ");
        System.out.println("                          - if not provided the default of OFF");
        System.out.println("                            is used                           ");
        System.out.println();
        System.out.println(" [ -l | --lb_policy ] <int>                              "); 
        System.out.println("                         the load balancing policy to be applied");
        System.out.println();
        System.out.println("                           Possible values               ");
        System.out.println();
        System.out.println("                             0 = Random                  ");
        System.out.println("                             1 = Round Robin             ");
        System.out.println("                             2 = FIFO                    ");
        System.out.println("                             3 = LIFO                    ");
        System.out.println("                                                         ");
        System.out.println("                         - if not provided the default of");
        System.out.println("                           Random is used if multiples   ");
        System.out.println("                           allowed                       ");
        System.out.println();        
        System.out.println(" [ -v | --act_ior ] <ior>                                ");  
        System.out.println("                         the IOR of the Activator to be  ");
        System.out.println("                         used for this server if         ");
        System.out.println("                         auto-activation is allowed      ");
        System.out.println("");
        System.out.println(" [ -e | --mode ] <mode>  the activation mode to be applied ");
        System.out.println();
        System.out.println("                             0 = NORMAL                    ");
        System.out.println("                             1 = MANUAL                    ");
        System.out.println("                             2 = PER_CLIENT                ");
        System.out.println("                             3 = AUTO_START                ");
        System.out.println();
        System.out.println("                          the default of 0 is used if this ");
        System.out.println("                          parameter is not provided        ");
        System.out.println();
        System.out.println("---------------------------------------------------------");
        System.out.println(" [ -r | --remove ]                                       ");
        System.out.println();
        System.out.println(" [ -n | --name ] <name>  Removes all servers from the    ");
        System.out.println("                         repository for the given name   ");
        System.out.println("                         Name is the name of the server to");
        System.out.println("                         be removed.                     ");
        System.out.println();
        System.out.println(" [ -p  | --poa ] <poa name> Not yet implemented          ");
        System.out.println("---------------------------------------------------------");
        System.out.println(" [ -d | --deactivate ] [ -s | --server ] <server id>     ");
        System.out.println("                         Forces the server with the given");
        System.out.println("                         id to be deactivated.           ");
        System.out.println("---------------------------------------------------------");
        System.out.println(" [ -k| --shutdown ]                                      ");
        System.out.println();
        System.out.println(" [ -n | --name ] <name>  The implementation name under   ");
        System.out.println("                         which the server to be shutdown ");
        System.out.println("                         is registered (mandatory).      ");
        System.out.println();
        System.out.println(" [ -p  | --poa ] <poa name> Not yet implemented          ");        
        System.out.println("");
        System.out.println(" <-s , --server>         The server ID of the server,    ");
        System.out.println("                         This must be provided if        ");
        System.out.println("                         multiple servers are registered ");
        System.out.println("---------------------------------------------------------");
        System.out.println(" [ -h | --help ]         Displays this help              ");
        System.out.println("---------------------------------------------------------");

        System.exit(0);
    }
}
