package org.jacorb.orb.giop;

/*
 *        JacORB - a free Java ORB
 *
 *   Copyright (C) 1997-2003  Gerald Brose.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.io.UnsupportedEncodingException;
import org.jacorb.util.Debug;
import org.omg.CORBA.DATA_CONVERSION;
import org.omg.CORBA.INTERNAL;
import sun.io.CharToByteConverter;
import sun.io.ConversionBufferFullException;
import sun.io.MalformedInputException;
import sun.io.UnknownCharacterException;


/**
 * <code>CTBConverter</code> is a class to convert char arrays to byte arrays
 * according to a chosen encoding. This version uses the sun.io classes to do
 * the conversion.
 *
 * @author <a href="mailto:rnc@prismtechnologies.com">Nick Cross</a>
 * @version 1.0
 */
public class CTBConverter
{
    /**
     * <code>converter</code> is the actual Sun converter.
     */
    private CharToByteConverter converter;


    /**
     * <code>CTBConverter</code> creates a new instance with a specific encoding.
     *
     * @param encoding a <code>String</code> value
     */
    public CTBConverter(String encoding)
    {
        try
        {
            converter = CharToByteConverter.getConverter(encoding);

            // When substitution mode is false, the converter
            // will throw exceptions when it encounters illegal
            // encodings.
            converter.setSubstitutionMode(false);
        }
        catch (UnsupportedEncodingException ex)
        {
            // Unknown encoding
            if (Debug.isDebugEnabled())
            {
                Debug.output(4, ex);
            }
            throw new DATA_CONVERSION(ex.getMessage());
        }
    }



    /**
     * <code>convert</code> returns the converted byte array.
     *
     * @param source a <code>char[]</code> value; the character array
     * @param offset an <code>int</code> value; the offset to start at
     * @return a <code>byte[]</code> value
     */
    public byte[] convert (char[] value, int offset)
    {
        byte []result = null;

        try
        {
            int numBytes = 0;
            int numChars = value.length - offset;

            result = new byte[numChars * converter.getMaxBytesPerChar()];

            converter.reset();

            // Convert the characters
            numBytes = converter.convert(value, offset, numChars,
                                   result, 0, result.length);

            // Converters must be flushed to finish up
            numBytes += converter.flush(result, 0, result.length);

            if (numBytes < result.length)
            {
                // A short format was used:  Trim the byte array.
                byte[] trimResult = new byte[numBytes];
                System.arraycopy(result, 0, trimResult, 0, numBytes);
                result = trimResult;
            }
        }
        catch (MalformedInputException ex)
        {
            // There were illegal Unicode char pairs
            if (Debug.isDebugEnabled())
            {
                Debug.output(4, ex);
            }
            throw new DATA_CONVERSION(ex.getMessage());
        }
        catch (UnknownCharacterException ex)
        {
            // A character doesn't map to the desired code set
            // CORBA formal 00-11-03.
            if (Debug.isDebugEnabled())
            {
                Debug.output(4, ex);
            }
            throw new DATA_CONVERSION(ex.getMessage());
        }
        catch (ConversionBufferFullException ex)
        {
            // If this happens, then the CharToByteConverter was lying
            // about the maximum bytes per char.
            if (Debug.isDebugEnabled())
            {
                Debug.output(4, ex);
            }
            throw new INTERNAL(ex.getMessage());
        }
        return result;
    }
}
