package org.jacorb.test.imrutility.imr;

import junit.framework.*;

import org.jacorb.imrutility.imr.*;
import org.jacorb.test.common.*;
import org.jacorb.util.Environment;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Properties;

/**
 * This class tests a straightforward case of starting the ImR
 * checking the contents (as out of the box).  Starting a 
 * server and checking registration has been successful, then
 * attempting to connect to the server which will involve locating
 * the server via the IMR
 */
public class BasicImRTest extends PersistentServerTestCase
{ 
    private String serverIOR;
    private final String ior = "server.ior";
    private final String home = System.getProperty("user.home");
    private final String separator = System.getProperty("file.separator");
    private RepositoryAdmin rep;
    private Process proc;
        
    public BasicImRTest(String name, PersistentServerSetup setup)
    {
        super(name, setup);
    }

    public static Test suite()
    {
        TestSuite suite = new TestSuite ("Basic ImR Test");
        
        Properties clientProps = new Properties();
               
        PersistentServerSetup setup =
            new PersistentServerSetup
            ( 
                suite,
                "org.jacorb.test.imrutility.imr.Server"
            );
 
        suite.addTest (new BasicImRTest ("testImRBasic", setup));
        
        // Return the setup, not the suite!
        return setup;
    }

    protected void tearDown()
    {
        System.err.println("********** tearDown for testImRBasic");
        try
        {
            if (proc != null)
            {  
                proc.destroy();
            }
         
            if (rep != null)
            {        
                rep.shutdown(false);
            }
        
            TestUtility.removeIMRFiles();
            TestUtility.removeFile(serverIOR);
            TestUtility.destroyImR();
        }
        catch (Exception ex)
        {
            /* don't do anything */
        }    
    }
        
    public void testImRBasic()
    {
        ImplInfo [] infos = null;
        StringBuffer command = null;
        ServerInfo [] sinfos = null;
        POAInfo [] poas = null;
        TestServer test = null;
        String [] args = null;
                              
        try
        {        
            TestUtility.removeIMRFiles();
        }
        catch (Exception ex)
        {
            /* Don't do anything as this is not a problem*/
        }

        //Start the IMR        
        try
        {             
            args = new String [0];
            rep = TestUtility.startImR(args, null);
            
            assertTrue("Failed to get ImR", rep != null);
        }
        catch(Exception ex)
        {
            fail("Exception getting ImR " + ex.getMessage());
        }                

        infos = rep.listAllImpls();
        
        /*Check no data in IMR imr.listallimpls should return empty list*/
        assertTrue
        (
            "Implementations registered at startup", 
            infos.length == 0
        );
        
        try
        {
            command = setup.getStartCommand();
            command.append(" -i ");
            serverIOR = home + separator + ior;
            command.append(serverIOR);
            
            proc = TestUtility.startServer(command);
        
            /* Wait for the IOR file to be written & Server registered*/
            Thread.sleep(2000);            
        }
        catch (IOException ioe)
        {
            fail("Got IOException starting server " + ioe.getMessage());
        }
        catch (Exception ex)
        {
            fail("Got Exception starting server " + ex.getMessage());
        }

        infos = rep.listAllImpls();
        
        assertTrue
        (
            "Incorrect number of Implementations registered", 
            infos != null && infos.length == 1
        );
 
        assertTrue
        (
            "Incorrect implementation name recorded",
            infos[0].name.impl().equals("test_imr")
        );
        
        assertTrue
        (
            "Multiples value incorrect",
            infos[0].multiples == false
        );
         
        assertTrue
        (
            "Load Balancing value incorrect",
            infos[0].lbPolicy.value() == LoadBalancingPolicy._RANDOM
        );

        try
        {
            sinfos = rep.listServers(infos[0].name);
                
            assertTrue
            (
                "Incorrect number of servers registered",
                sinfos != null && sinfos.length == 1
            );
        
            assertTrue
            (
                "ServerStatus incorrect",
                sinfos[0].state.value() == ServerState._ACTIVE
            );
        
            assertTrue
            (
                "Server is null",
                sinfos[0].srv != null
            );
        }
        catch (Exception ex)
        {
            fail("Exception listing servers");
        }
                
        try
        {
            poas = rep.listPOAs(sinfos[0].id);            
        
            assertTrue
            (
                "Incorrect number of POAs registered",
                poas != null && poas.length == 1
            );
        
            assertTrue
            (
                "POA name incorrect",
                poas[0].name.equals("test_imr/TestIMRPOA")
            );
        
            assertTrue
            (
                "Load Balancing value incorrect",
                poas[0].lbPolicy.value() == LoadBalancingPolicy._RANDOM
            );        
        }
        catch (Exception ex)
        {
            fail("Exception listing POAs");
        }
        
        try
        {        
            BufferedReader br =
                new BufferedReader(new FileReader(serverIOR));

            // get object reference from command-line argument file
            org.omg.CORBA.Object obj =
                TestUtility.getImRORB().string_to_object(br.readLine());

            br.close();

            //narrow to correct interface
            test = TestServerHelper.narrow(obj);
        }
        catch (Exception ex)
        {
            fail("Exception reading server IOR");
        }

        try
        {
            test.ping();
        }
        catch (Exception ex)
        {
            fail("Exception pinging server");
        }
    }            
}
