package org.jacorb.test.imrutility.imr;


import junit.framework.*;

import org.jacorb.imrutility.imr.*;
import org.jacorb.util.Environment;

import java.io.IOException;
import java.util.Properties;

/**
 * This class tests the registration of a server where
 * 
 *   1. There is an ImR data file with details that result in
 *      implementation details being recorded
 *   2. The server is registered, fails and then restarts and
 *      the original server details are still recorded with 
 *      the ImR.
 * 
 * In both circumstances the server should be registered 
 * successfully with the ImR.
 */
public class ServerRestartTest extends PersistentServerTestCase
{
    private String serverIOR;
    private final String ior = "server.ior";
    private final String home = System.getProperty("user.home");
    private final String separator = System.getProperty("file.separator");
    private RepositoryAdmin rep;
    private Process proc;
        
    public ServerRestartTest(String name, PersistentServerSetup setup)
    {
        super(name, setup);
    }

    public static Test suite()
    {
        TestSuite suite = new TestSuite ("Server Restart Test");
        
        Properties clientProps = new Properties();
               
        PersistentServerSetup setup =
            new PersistentServerSetup
            ( 
                suite,
                "org.jacorb.test.imrutility.imr.Server"
            );
 
        suite.addTest(new ServerRestartTest("serverRestart", setup));
        
        // Return the setup, not the suite!
        return setup;
    }

    protected void tearDown()
    {
                System.err.println("********** tearDown for serverRestart");
               
        try
        {
            if (proc != null)
            {
                proc.destroy();
            }
             
            if (rep != null)
            {
                rep.shutdown(false);
            }
            
            /* Only remove the IOR file - leave the data file for the next 
             * test
             */
            TestUtility.removeFile
                (Environment.getProperty("jacorb.imr.ior_file"));      
                
            TestUtility.destroyImR();    
            Thread.sleep(1000);  
        }
        catch (Exception ex)
        {
            // Do nothing
        }           
    }
    
    /**
     * Tests restart of the server where the server is running, fails then
     * restarts.  The ImR continues to hold registration details of the
     * original server but is not configured to ping the Server in any
     * way and so is unaware of the original failure.  The server should
     * be registered successfully on restart as the original server is
     * not running
     */
    public void serverRestart()
    {
        ImplInfo [] infos = null;
        StringBuffer command = null;
        ServerInfo [] sinfos = null;
        POAInfo [] poas = null;
        TestServer test = null;
        String [] args = null;
                              
        try
        {        
            TestUtility.removeIMRFiles();
        }
        catch (Exception ex)
        {
            /* Don't do anything as this is not a problem*/
        }

        //Start the IMR        
        try
        {             
            args = new String[0];
            rep = TestUtility.startImR(args, null);
            
            assertTrue("Failed to get ImR", rep != null);
        }
        catch(Exception ex)
        {
            fail("Exception getting ImR" + ex.getMessage());
        }                

        /* Start then stop the server */
        try
        {
            command = setup.getStartCommand();
            command.append(" -i ");
            serverIOR = home + separator + ior;
            command.append(serverIOR);
            
            proc = TestUtility.startServer(command);
        
            /* Wait for the IOR file to be written & Server registered*/
            Thread.sleep(2000);            
            
            /* Stop this server straight away */
            proc.destroy();
        }
        catch (IOException ioe)
        {
            fail("Got IOException starting server " + ioe.getMessage());
        }
        catch (Exception ex)
        {
            fail(ex.getMessage());
        }

        /* Restart the server */
        try
        {
            command = setup.getStartCommand();
            command.append(" -i ");
            serverIOR = home + separator + ior;
            command.append(serverIOR);
            
            proc = TestUtility.startServer(command);
        
            /* Wait for the IOR file to be written & Server registered*/
            Thread.sleep(2000);                        
        }
        catch (IOException ioe)
        {
            fail("Got IOException starting server " + ioe.getMessage());
        }
        catch (Exception ex)
        {
            fail(ex.getMessage());
        }

        try
        {
            ServerName sname = new ServerName();
            sname.impl("test_imr");
            
            sinfos = rep.listServers(sname);
                
            assertTrue
            (
                "Incorrect number of servers registered",
                sinfos != null && sinfos.length == 1
            );
        
            assertTrue
            (
                "ServerStatus incorrect",
                sinfos[0].state.value() == ServerState._ACTIVE
            );
        
            assertTrue
            (
                "Server is null",
                sinfos[0].srv != null
            );
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            fail("Exception listing servers");
        }               
    }
}



